@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,13 +19,13 @@
 
 import ast
 
-from nuitka.Options import hasPythonFlagNoAsserts, hasPythonFlagNoDocstrings
+from nuitka.Options import hasPythonFlagNoAsserts, hasPythonFlagNoDocStrings
 from nuitka.tree.TreeHelpers import getKind
 
 doc_having = tuple(
-    getattr(ast, cand)
-    for cand in ("FunctionDef", "ClassDef", "AsyncFunctionDef")
-    if hasattr(ast, cand)
+    getattr(ast, candidate)
+    for candidate in ("FunctionDef", "ClassDef", "AsyncFunctionDef")
+    if hasattr(ast, candidate)
 )
 
 
@@ -43,24 +43,37 @@ def compileSourceToBytecode(source_code, filename):
     # Prepare compile call with AST tree.
     tree = ast.parse(source_code, filename)
 
-    # Do we need to remove docstrings.
-    remove_docstrings_from_tree = hasPythonFlagNoDocstrings()
+    # Do we need to remove doc strings.
+    remove_doc_strings_from_tree = hasPythonFlagNoDocStrings()
 
     # For Python2, we need to do this manually.
     remove_asserts_from_tree = hasPythonFlagNoAsserts() and str is bytes
 
-    if remove_docstrings_from_tree or remove_asserts_from_tree:
+    if remove_doc_strings_from_tree or remove_asserts_from_tree:
         # Module level docstring.
-        if remove_docstrings_from_tree:
+        if remove_doc_strings_from_tree:
             _removeDocFromBody(tree)
 
         for node in ast.walk(tree):
             if remove_asserts_from_tree:
-                if type(node) is ast.Name and node.id == "__debug__":
-                    node.id = "False"
+                node_type = type(node)
+
+                if node_type is ast.Name:
+                    if node.id == "__debug__":
+                        node.id = "False"
+
+                elif node_type is ast.Assert:
+                    # Cannot really remove the assertion node easily, lets just replace it with
+                    # "assert 1" and remove the assert msg. Probably not worth more effort for
+                    # Python2 at this time.
+                    node.test = ast.Num()
+                    node.test.n = 1
+                    node.test.lineno = node.lineno
+                    node.test.col_offset = node.col_offset
+                    node.msg = None
 
             # Check if it's a docstring having node type.
-            if remove_docstrings_from_tree and isinstance(node, doc_having):
+            if remove_doc_strings_from_tree and isinstance(node, doc_having):
                 _removeDocFromBody(node)
 
     if str is bytes:
