@@ -379,12 +379,18 @@ Error, '--nofollow-import-to' takes only module names or patterns, not directory
     output_filename = getOutputFilename()
 
     if output_filename is not None:
-        if (isStandaloneMode() and not isOnefileMode()) or shallMakeModule():
+        if shallMakeModule():
             Tracing.options_logger.sysexit(
                 """\
-Error, may only specify output filename for acceleration and onefile mode,
-but not for module mode where filenames are mandatory, and not for
-standalone where there is a sane default used inside the dist folder."""
+Error, may not module mode where filenames and modules matching are
+mandatory."""
+            )
+        elif (
+            isStandaloneMode() and os.path.basename(output_filename) != output_filename
+        ):
+            Tracing.options_logger.sysexit(
+                """\
+Error, output filename for standalone cannot contain a directory part."""
             )
 
         output_dir = os.path.dirname(output_filename) or "."
@@ -830,7 +836,7 @@ but errors may happen."""
             """\
 Using very slow fallback for ordered sets, please install '%s' \
 PyPI package for best Python compile time performance."""
-            % ("ordered-set" if python_version >= 0x3A0 else "orderedset")
+            % ("ordered-set" if python_version >= 0x370 else "orderedset")
         )
 
     if shallUsePythonDebug() and not isDebugPython():
@@ -878,6 +884,22 @@ to work."""
 Error, to compile a package, specify its directory but, not the '__init__.py'."""
             )
 
+    if (
+        isWin32Windows()
+        and 0x340 <= python_version < 0x380
+        and not shallDisableConsoleWindow()
+    ):
+        Tracing.general.warning(
+            """\
+On Windows, support for input/output on the console Windows, does \
+not work on non-UTF8 systems, unless Python 3.8 or higher is used \
+but this is %s, so please consider upgrading, or disabling the \
+console window for deployment.
+"""
+            % python_version_str,
+            mnemonic="old-python-windows-console",
+        )
+
 
 def isVerbose():
     """:returns: bool derived from ``--verbose``"""
