@@ -218,30 +218,19 @@ class TempMixin(object):
         # For finally handlers of Python3, which have conditions on assign and
         # use.
         if preserver_id not in self.preserver_variable_declaration:
+            needs_init = Options.is_debug and python_version >= 0x300
 
-            debug = Options.is_debug and python_version >= 0x300
-
-            if debug:
-                preserver_obj_init = "NULL"
+            if needs_init:
+                preserver_obj_init = "Nuitka_ExceptionStackItem_Empty"
             else:
                 preserver_obj_init = None
 
-            self.preserver_variable_declaration[preserver_id] = (
-                self.variable_storage.addVariableDeclarationTop(
-                    "PyObject *",
-                    "exception_preserved_type_%d" % preserver_id,
-                    preserver_obj_init,
-                ),
-                self.variable_storage.addVariableDeclarationTop(
-                    "PyObject *",
-                    "exception_preserved_value_%d" % preserver_id,
-                    preserver_obj_init,
-                ),
-                self.variable_storage.addVariableDeclarationTop(
-                    "PyTracebackObject *",
-                    "exception_preserved_tb_%d" % preserver_id,
-                    preserver_obj_init,
-                ),
+            self.preserver_variable_declaration[
+                preserver_id
+            ] = self.variable_storage.addVariableDeclarationTop(
+                "struct Nuitka_ExceptionStackItem",
+                "exception_preserved_%d" % preserver_id,
+                preserver_obj_init,
             )
 
         return self.preserver_variable_declaration[preserver_id]
@@ -293,6 +282,7 @@ CodeObjectHandle = collections.namedtuple(
     "CodeObjectHandle",
     (
         "co_name",
+        "co_qualname",
         "co_kind",
         "co_varnames",
         "co_argcount",
@@ -322,6 +312,7 @@ class CodeObjectsMixin(object):
         key = CodeObjectHandle(
             co_filename=code_object.getFilename(),
             co_name=code_object.getCodeObjectName(),
+            co_qualname=code_object.getCodeObjectQualname(),
             line_number=code_object.getLineNumber(),
             co_varnames=code_object.getVarNames(),
             co_argcount=code_object.getArgumentCount(),
@@ -854,6 +845,10 @@ class PythonModuleContext(
         del self.function_table_entries
         return result
 
+    @staticmethod
+    def getContextObjectName():
+        return None
+
 
 class PythonFunctionContext(
     FrameDeclarationsMixin,
@@ -907,6 +902,10 @@ class PythonFunctionContext(
 
 
 class PythonFunctionDirectContext(PythonFunctionContext):
+    @staticmethod
+    def getContextObjectName():
+        return None
+
     def isForDirectCall(self):
         return True
 
@@ -924,7 +923,8 @@ class PythonGeneratorObjectContext(PythonFunctionContext):
     def isForCreatedFunction(self):
         return False
 
-    def getContextObjectName(self):
+    @staticmethod
+    def getContextObjectName():
         return "generator"
 
     def getGeneratorReturnValueName(self):
@@ -935,16 +935,22 @@ class PythonGeneratorObjectContext(PythonFunctionContext):
 
 
 class PythonCoroutineObjectContext(PythonGeneratorObjectContext):
-    def getContextObjectName(self):
+    @staticmethod
+    def getContextObjectName():
         return "coroutine"
 
 
 class PythonAsyncgenObjectContext(PythonGeneratorObjectContext):
-    def getContextObjectName(self):
+    @staticmethod
+    def getContextObjectName():
         return "asyncgen"
 
 
 class PythonFunctionCreatedContext(PythonFunctionContext):
+    @staticmethod
+    def getContextObjectName():
+        return None
+
     def isForDirectCall(self):
         return False
 
@@ -1072,6 +1078,9 @@ class PythonFunctionOutlineContext(
         # The outline is only accessible via its parent.
         return self.parent.mayRecurse()
 
+    def getLocalsDictNames(self):
+        return self.parent.getLocalsDictNames()
+
     def addLocalsDictName(self, locals_dict_name):
         return self.parent.addLocalsDictName(locals_dict_name)
 
