@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -55,7 +55,15 @@ def _recurseTo(module_name, module_filename, module_kind):
     return module, is_added
 
 
-def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
+def recurseTo(
+    signal_change,
+    module_name,
+    module_filename,
+    module_kind,
+    using_module,
+    source_ref,
+    reason,
+):
     try:
         module = ImportCache.getImportedModuleByNameAndPath(
             module_name, module_filename
@@ -68,6 +76,8 @@ def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
             module_filename=module_filename,
             module_name=module_name,
             module_kind=module_kind,
+            using_module=using_module,
+            source_ref=source_ref,
         )
 
         module, added_flag = _recurseTo(
@@ -82,7 +92,21 @@ def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
     return module
 
 
+_recursion_decision_cache = {}
+
+
 def decideRecursion(module_filename, module_name, module_kind, extra_recursion=False):
+    key = module_filename, module_name, module_kind, extra_recursion
+
+    if key not in _recursion_decision_cache:
+        _recursion_decision_cache[key] = _decideRecursion(
+            module_filename, module_name, module_kind, extra_recursion
+        )
+
+    return _recursion_decision_cache[key]
+
+
+def _decideRecursion(module_filename, module_name, module_kind, extra_recursion):
     # Many branches, which make decisions immediately, by returning
     # pylint: disable=too-many-branches,too-many-return-statements
     if module_name == "__main__":
@@ -177,24 +201,8 @@ def decideRecursion(module_filename, module_name, module_kind, extra_recursion=F
 
 
 def considerFilename(module_filename):
-    module_filename = os.path.normpath(module_filename)
-
-    if os.path.isdir(module_filename):
-        module_filename = os.path.abspath(module_filename)
-
-        module_name = os.path.basename(module_filename)
-
-        return module_filename, module_name
-    elif module_filename.endswith(".py"):
-        module_name = os.path.basename(module_filename)[:-3]
-
-        return module_filename, module_name
-    elif module_filename.endswith(".pyw"):
-        module_name = os.path.basename(module_filename)[:-4]
-
-        return module_filename, module_name
-    else:
-        return None
+    module_name, module_kind = getModuleNameAndKindFromFilename(module_filename)
+    return None if module_kind == "extension" else module_filename, module_name
 
 
 def isSameModulePath(path1, path2):
@@ -206,9 +214,61 @@ def isSameModulePath(path1, path2):
     return os.path.abspath(path1) == os.path.abspath(path2)
 
 
-def checkPluginSinglePath(plugin_filename, module_package):
+def _addIncludedModule(module):
     # Many branches, for the decision is very complex, pylint: disable=too-many-branches
 
+    if Options.isShowInclusion():
+        recursion_logger.info(
+            "Included '%s' as '%s'."
+            % (
+                module.getFullName(),
+                module,
+            )
+        )
+
+    ImportCache.addImportedModule(module)
+
+    if module.isCompiledPythonPackage() or module.isUncompiledPythonPackage():
+        package_filename = module.getFilename()
+
+        if os.path.isdir(package_filename):
+            # Must be a namespace package.
+            assert python_version >= 0x300
+
+            package_dir = package_filename
+
+            # Only include it, if it contains actual modules, which will
+            # recurse to this one and find it again.
+        else:
+            package_dir = os.path.dirname(package_filename)
+
+            # Real packages will always be included.
+            ModuleRegistry.addRootModule(module)
+
+        if Options.isShowInclusion():
+            recursion_logger.info("Package directory '%s'." % package_dir)
+
+        for sub_path, sub_filename in listDir(package_dir):
+            if sub_filename in ("__init__.py", "__pycache__"):
+                continue
+
+            if Importing.isPackageDir(sub_path) and not os.path.exists(
+                sub_path + ".py"
+            ):
+                checkPluginSinglePath(sub_path, module_package=module.getFullName())
+            elif sub_path.endswith(".py"):
+                checkPluginSinglePath(sub_path, module_package=module.getFullName())
+
+    elif module.isCompiledPythonModule() or module.isUncompiledPythonModule():
+        ModuleRegistry.addRootModule(module)
+    elif module.isPythonExtensionModule():
+        if Options.isStandaloneMode():
+            ModuleRegistry.addRootModule(module)
+    else:
+        assert False, module
+
+
+def checkPluginSinglePath(plugin_filename, module_package):
     # The importing wants these to be unique.
     plugin_filename = os.path.abspath(plugin_filename)
 
@@ -244,74 +304,28 @@ def checkPluginSinglePath(plugin_filename, module_package):
                 module_filename=plugin_filename,
                 module_name=module_name,
                 module_kind=module_kind,
+                using_module=None,
+                source_ref=None,
                 reason=reason,
             )
 
             if module:
-                if Options.isShowInclusion():
-                    recursion_logger.info(
-                        "Included '%s' as '%s'."
-                        % (
-                            module.getFullName(),
-                            module,
-                        )
-                    )
-
-                ImportCache.addImportedModule(module)
-
-                if module.isCompiledPythonPackage():
-                    package_filename = module.getFilename()
-
-                    if os.path.isdir(package_filename):
-                        # Must be a namespace package.
-                        assert python_version >= 0x300
-
-                        package_dir = package_filename
-
-                        # Only include it, if it contains actual modules, which will
-                        # recurse to this one and find it again.
-                    else:
-                        package_dir = os.path.dirname(package_filename)
-
-                        # Real packages will always be included.
-                        ModuleRegistry.addRootModule(module)
-
-                    if Options.isShowInclusion():
-                        recursion_logger.info("Package directory '%s'." % package_dir)
-
-                    for sub_path, sub_filename in listDir(package_dir):
-                        if sub_filename in ("__init__.py", "__pycache__"):
-                            continue
-
-                        assert sub_path != plugin_filename
-
-                        if Importing.isPackageDir(sub_path) and not os.path.exists(
-                            sub_path + ".py"
-                        ):
-                            checkPluginSinglePath(
-                                sub_path, module_package=module.getFullName()
-                            )
-                        elif sub_path.endswith(".py"):
-                            checkPluginSinglePath(
-                                sub_path, module_package=module.getFullName()
-                            )
-
-                elif module.isCompiledPythonModule():
-                    ModuleRegistry.addRootModule(module)
-                elif module.isPythonExtensionModule():
-                    if Options.isStandaloneMode():
-                        ModuleRegistry.addRootModule(module)
-
+                _addIncludedModule(module)
             else:
                 recursion_logger.warning(
                     "Failed to include module from '%s'." % plugin_filename
                 )
+        else:
+            recursion_logger.warning(
+                "Not allowed to include module '%s' due to '%s'."
+                % (module_name, reason)
+            )
 
 
 def checkPluginPath(plugin_filename, module_package):
     if Options.isShowInclusion():
         recursion_logger.info(
-            "Checking top level plug-in path %s %s" % (plugin_filename, module_package)
+            "Checking top level path '%s' '%s'." % (plugin_filename, module_package)
         )
 
     plugin_info = considerFilename(module_filename=plugin_filename)
@@ -328,10 +342,12 @@ def checkPluginPath(plugin_filename, module_package):
                     checkPluginSinglePath(sub_path, module_package=None)
         else:
             recursion_logger.warning(
-                "Failed to include module from %r." % plugin_info[0]
+                "Failed to include module from '%s'." % plugin_info[0]
             )
     else:
-        recursion_logger.warning("Failed to recurse to directory %r." % plugin_filename)
+        recursion_logger.warning(
+            "Failed to recurse to directory '%s'." % plugin_filename
+        )
 
 
 def checkPluginFilenamePattern(pattern):
@@ -353,22 +369,33 @@ def checkPluginFilenamePattern(pattern):
         checkPluginSinglePath(filename, module_package=None)
 
     if not found:
-        recursion_logger.warning("Didn't match any files against pattern %r." % pattern)
+        recursion_logger.warning(
+            "Didn't match any files against pattern '%s'." % pattern
+        )
 
 
 def _addParentPackageUsages(using_module, module_name, signal_change, source_ref):
     for parent_package_name in module_name.getParentPackageNames():
-        _parent_package_name, parent_package_filename, _finding = locateModule(
-            module_name=parent_package_name, parent_package=None, level=0
-        )
+        (
+            _parent_package_name,
+            parent_package_filename,
+            package_module_kind,
+            finding,
+        ) = locateModule(module_name=parent_package_name, parent_package=None, level=0)
+
+        if parent_package_filename is None:
+            recursion_logger.sysexit(
+                "Error, failed to locate parent package file for '%s' parent of '%s' (used by '%s') module (%s)"
+                % (
+                    parent_package_name.asString(),
+                    module_name.asString(),
+                    using_module.getFullName().asString(),
+                    finding,
+                )
+            )
 
-        assert parent_package_filename is not None, parent_package_name
         assert _parent_package_name == parent_package_name
 
-        _parent_package_name, package_module_kind = getModuleNameAndKindFromFilename(
-            parent_package_filename
-        )
-
         _decision, reason = decideRecursion(
             module_filename=parent_package_filename,
             module_name=parent_package_name,
@@ -380,6 +407,8 @@ def _addParentPackageUsages(using_module, module_name, signal_change, source_ref
             module_name=parent_package_name,
             module_filename=parent_package_filename,
             module_kind=package_module_kind,
+            using_module=using_module,
+            source_ref=source_ref,
             reason=reason,
         )
 
@@ -393,62 +422,65 @@ def _addParentPackageUsages(using_module, module_name, signal_change, source_ref
 
 
 def considerUsedModules(module, signal_change):
-    for (
-        used_module_name,
-        used_module_filename,
-        finding,
-        level,
-        source_ref,
-    ) in module.getUsedModules():
-        if finding == "not-found":
+    for used_module in module.getUsedModules():
+        if used_module.finding == "not-found":
             Importing.warnAbout(
                 importing=module,
-                source_ref=source_ref,
-                module_name=used_module_name,
-                level=level,
+                source_ref=used_module.source_ref,
+                module_name=used_module.module_name,
+                level=used_module.level,
             )
 
+        # Nothing was found here
+        if used_module.filename is None:
+            continue
+
         try:
-            if used_module_filename is None:
-                continue
-
-            _module_name, module_kind = getModuleNameAndKindFromFilename(
-                used_module_filename
-            )
-
             decision, reason = decideRecursion(
-                module_filename=used_module_filename,
-                module_name=used_module_name,
-                module_kind=module_kind,
+                module_filename=used_module.filename,
+                module_name=used_module.module_name,
+                module_kind=used_module.module_kind,
             )
 
             if decision:
                 _addParentPackageUsages(
                     using_module=module,
-                    module_name=used_module_name,
+                    module_name=used_module.module_name,
                     signal_change=signal_change,
-                    source_ref=source_ref,
+                    source_ref=used_module.source_ref,
                 )
 
-                used_module = recurseTo(
+                new_module = recurseTo(
                     signal_change=signal_change,
-                    module_name=used_module_name,
-                    module_filename=used_module_filename,
-                    module_kind=module_kind,
+                    module_name=used_module.module_name,
+                    module_filename=used_module.filename,
+                    module_kind=used_module.module_kind,
+                    source_ref=used_module.source_ref,
+                    using_module=module,
                     reason=reason,
                 )
 
                 addUsedModule(
-                    module=used_module,
+                    module=new_module,
                     using_module=module,
                     usage_tag="import",
                     reason=reason,
-                    source_ref=source_ref,
+                    source_ref=used_module.source_ref,
                 )
         except NuitkaForbiddenImportEncounter as e:
             recursion_logger.sysexit(
                 "Error, forbidden import of '%s' in module '%s' at '%s' encountered."
-                % (e, module.getFullName().asString(), source_ref.getAsString())
+                % (
+                    e,
+                    module.getFullName().asString(),
+                    used_module.source_ref.getAsString(),
+                )
             )
 
-    Plugins.considerImplicitImports(module=module, signal_change=signal_change)
+    try:
+        Plugins.considerImplicitImports(module=module, signal_change=signal_change)
+    except NuitkaForbiddenImportEncounter as e:
+        recursion_logger.sysexit(
+            "Error, forbidden import of '%s' done implicitly by module '%s'."
+            % (e, module.getFullName().asString())
+        )
