@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,16 +26,20 @@ good.
 
 from nuitka.PythonVersions import python_version
 
-from .ExpressionBases import (
-    ExpressionBuiltinSingleArgBase,
-    ExpressionChildrenHavingBase,
-)
-from .NodeBases import StatementChildHavingBase
+from .BuiltinLenNodes import ExpressionBuiltinLen
+from .ExpressionBases import ExpressionBuiltinSingleArgBase
+from .ExpressionBasesGenerated import ExpressionBuiltinIter2Base
 from .NodeMakingHelpers import (
     makeRaiseExceptionReplacementStatement,
+    makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue,
     wrapExpressionWithSideEffects,
 )
 from .shapes.StandardShapes import tshape_iterator
+from .StatementBasesGenerated import (
+    StatementSpecialUnpackCheckBase,
+    StatementSpecialUnpackCheckFromIteratedBase,
+)
+from .VariableRefNodes import ExpressionTempVariableRef
 
 
 class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
@@ -43,11 +47,12 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
 
     simulator = iter
 
-    def computeExpression(self, trace_collection):
-        trace_collection.initIteratorValue(self)
+    @staticmethod
+    def isExpressionBuiltinIter1():
+        return True
 
-        value = self.subnode_value
-        return value.computeExpressionIter1(
+    def computeExpression(self, trace_collection):
+        return self.subnode_value.computeExpressionIter1(
             iter_node=self, trace_collection=trace_collection
         )
 
@@ -75,12 +80,13 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
                 "Predicted 'next' value from iteration.",
             )
 
-        # TODO: This is only true for a few value types, use type shape to tell if
+        # TODO: This is only relevant for a few value types, use type shape to tell if
         # it might escape or raise.
         self.onContentEscapes(trace_collection)
 
-        # Any code could be run, note that.
-        trace_collection.onControlFlowEscape(self)
+        if value.mayHaveSideEffectsNext():
+            # Any code could be run, note that.
+            trace_collection.onControlFlowEscape(self)
 
         # Any exception may be raised.
         trace_collection.onExceptionRaiseExit(BaseException)
@@ -153,6 +159,28 @@ class ExpressionBuiltinIter1(ExpressionBuiltinSingleArgBase):
 class ExpressionBuiltinIterForUnpack(ExpressionBuiltinIter1):
     kind = "EXPRESSION_BUILTIN_ITER_FOR_UNPACK"
 
+    def computeExpression(self, trace_collection):
+        result = self.subnode_value.computeExpressionIter1(
+            iter_node=self, trace_collection=trace_collection
+        )
+
+        result_node = result[0]
+
+        # Rewrite exceptions to correct message.
+        if (
+            result_node.isExpressionRaiseException()
+            and result_node.subnode_exception_type.isExpressionBuiltinExceptionRef()
+            and result_node.subnode_exception_type.getExceptionName() == "TypeError"
+        ):
+            return makeRaiseTypeErrorExceptionReplacementFromTemplateAndValue(
+                template="cannot unpack non-iterable %s object",
+                operation="iter",
+                original_node=self,
+                value_node=self.subnode_value,
+            )
+
+        return result
+
     @staticmethod
     def simulator(value):
         try:
@@ -163,87 +191,99 @@ class ExpressionBuiltinIterForUnpack(ExpressionBuiltinIter1):
             )
 
 
-class StatementSpecialUnpackCheck(StatementChildHavingBase):
+class StatementSpecialUnpackCheckFromIterated(
+    StatementSpecialUnpackCheckFromIteratedBase
+):
+    kind = "STATEMENT_SPECIAL_UNPACK_CHECK_FROM_ITERATED"
+
+    named_children = ("iterated_length",)
+    node_attributes = ("count",)
+    auto_compute_handling = "operation"
+
+    def computeStatementOperation(self, trace_collection):
+        if self.subnode_iterated_length.isCompileTimeConstant():
+            iterated_length_value = (
+                self.subnode_iterated_length.getCompileTimeConstant()
+            )
+
+            if iterated_length_value <= self.count:
+                return (
+                    None,
+                    "new_statements",
+                    lambda: "Determined iteration length check to be always true, because %d <= %d."
+                    % (iterated_length_value, self.count),
+                )
+            else:
+                result = makeRaiseExceptionReplacementStatement(
+                    statement=self,
+                    exception_type="ValueError",
+                    exception_value="too many values to unpack"
+                    if python_version < 0x300
+                    else "too many values to unpack (expected %d)" % self.count,
+                )
+
+                trace_collection.onExceptionRaiseExit(TypeError)
+
+                return (
+                    result,
+                    "new_raise",
+                    """\
+Determined iteration end check to always raise.""",
+                )
+
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        return self, None, None
+
+
+def makeStatementSpecialUnpackCheckFromIterated(
+    tmp_iterated_variable, count, source_ref
+):
+    return StatementSpecialUnpackCheckFromIterated(
+        iterated_length=ExpressionBuiltinLen(
+            ExpressionTempVariableRef(
+                variable=tmp_iterated_variable, source_ref=source_ref
+            ),
+            source_ref=source_ref,
+        ),
+        count=count,
+        source_ref=source_ref,
+    )
+
+
+class StatementSpecialUnpackCheck(StatementSpecialUnpackCheckBase):
     kind = "STATEMENT_SPECIAL_UNPACK_CHECK"
 
-    named_child = "iterator"
-
-    __slots__ = ("count",)
-
-    def __init__(self, iterator, count, source_ref):
-        StatementChildHavingBase.__init__(self, value=iterator, source_ref=source_ref)
-
-        self.count = int(count)
-
-    def getDetails(self):
-        return {"count": self.getCount()}
+    named_children = ("iterator",)
+    node_attributes = ("count",)
+    auto_compute_handling = "operation"
 
     def getCount(self):
         return self.count
 
-    def computeStatement(self, trace_collection):
-        iterator = trace_collection.onExpression(self.subnode_iterator)
+    def computeStatementOperation(self, trace_collection):
+        iterator = self.subnode_iterator
 
-        if iterator.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(BaseException)
+        if iterator.isExpressionTempVariableRef():
+            iteration_source_node = iterator.variable_trace.getIterationSourceNode()
 
-        if iterator.willRaiseException(BaseException):
-            from .NodeMakingHelpers import (
-                makeStatementExpressionOnlyReplacementNode,
-            )
+            if iteration_source_node is not None:
+                if iteration_source_node.parent.isStatementAssignmentVariableIterator():
+                    iterator_assign_node = iteration_source_node.parent
 
-            result = makeStatementExpressionOnlyReplacementNode(
-                expression=iterator, node=self
-            )
+                    if iterator_assign_node.tmp_iterated_variable is not None:
+                        result = makeStatementSpecialUnpackCheckFromIterated(
+                            tmp_iterated_variable=iterator_assign_node.tmp_iterated_variable,
+                            count=self.count,
+                            source_ref=self.source_ref,
+                        )
 
-            return (
-                result,
-                "new_raise",
-                """\
-Explicit raise already raises implicitly building exception type.""",
-            )
-
-        if (
-            iterator.isExpressionTempVariableRef()
-            and iterator.variable_trace.isAssignTrace()
-        ):
-
-            iterator = iterator.variable_trace.getAssignNode().subnode_source
-
-            current_index = trace_collection.getIteratorNextCount(iterator)
-        else:
-            current_index = None
-
-        if current_index is not None:
-            iter_length = iterator.getIterationLength()
-
-            if iter_length is not None:
-                # Remove the check if it can be decided at compile time.
-                if current_index == iter_length:
-                    return (
-                        None,
-                        "new_statements",
-                        """\
-Determined iteration end check to be always true.""",
-                    )
-                else:
-                    result = makeRaiseExceptionReplacementStatement(
-                        statement=self,
-                        exception_type="ValueError",
-                        exception_value="too many values to unpack"
-                        if python_version < 0x300
-                        else "too many values to unpack (expected %d)"
-                        % self.getCount(),
-                    )
-
-                    trace_collection.onExceptionRaiseExit(TypeError)
-
-                    return (
-                        result,
-                        "new_raise",
-                        """\
-Determined iteration end check to always raise.""",
-                    )
+                        return trace_collection.computedStatementResult(
+                            result,
+                            change_tags="new_statements",
+                            change_desc=lambda: "Iterator check of changed to iterated size check using '%s'."
+                            % iterator_assign_node.tmp_iterated_variable.getName(),
+                        )
 
         trace_collection.onExceptionRaiseExit(BaseException)
 
@@ -254,29 +294,21 @@ Determined iteration end check to always raise.""",
         return "iteration check statement"
 
 
-class ExpressionBuiltinIter2(ExpressionChildrenHavingBase):
+class ExpressionBuiltinIter2(ExpressionBuiltinIter2Base):
     kind = "EXPRESSION_BUILTIN_ITER2"
 
     named_children = ("callable_arg", "sentinel")
 
-    def __init__(self, callable_arg, sentinel, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values={"callable_arg": callable_arg, "sentinel": sentinel},
-            source_ref=source_ref,
-        )
+    auto_compute_handling = "final"
+
+    # TODO: The "callable" be investigated in a non-final
+    # "auto_compute_handling" here, as maybe it is not really callable.
 
     @staticmethod
     def getTypeShape():
-        # TODO: This could be more specific, this one is a fixed thing!
+        # TODO: This could be more specific, this one is a fixed well known thing!
         return tshape_iterator
 
-    def computeExpression(self, trace_collection):
-        # TODO: The "callable" should be investigated here, maybe it is not
-        # really callable, or raises an exception.
-
-        return self, None, None
-
     def computeExpressionIter1(self, iter_node, trace_collection):
         return self, "new_builtin", "Eliminated useless iterator creation."
 
