@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -51,7 +51,7 @@ class ExpressionBuiltinRefBase(CompileTimeConstantExpressionBase):
     __slots__ = ("builtin_name",)
 
     def __init__(self, builtin_name, source_ref):
-        CompileTimeConstantExpressionBase.__init__(self, source_ref=source_ref)
+        CompileTimeConstantExpressionBase.__init__(self, source_ref)
 
         self.builtin_name = builtin_name
 
@@ -82,7 +82,7 @@ def makeExpressionBuiltinTypeRef(builtin_name, source_ref):
     )
 
 
-quick_names = {"None": None, "True": True, "False": False, "Ellipsis": Ellipsis}
+quick_names = {"None": None, "True": True, "False": False}
 
 
 def makeExpressionBuiltinRef(builtin_name, locals_scope, source_ref):
@@ -150,6 +150,16 @@ class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
 
         return new_node, tags, message
 
+    def computeExpressionCallViaVariable(
+        self, call_node, variable_ref_node, call_args, call_kw, trace_collection
+    ):
+        return self.computeExpressionCall(
+            call_node=call_node,
+            call_args=call_args,
+            call_kw=call_kw,
+            trace_collection=trace_collection,
+        )
+
     @staticmethod
     def isKnownToBeIterable(count):
         # TODO: Why yes, some may be, could be told here.
@@ -183,7 +193,7 @@ class ExpressionBuiltinAnonymousRef(ExpressionBuiltinRefBase):
     __slots__ = ()
 
     def __init__(self, builtin_name, source_ref):
-        assert builtin_name in builtin_anon_names, builtin_name
+        assert builtin_name in builtin_anon_names, (builtin_name, source_ref)
 
         ExpressionBuiltinRefBase.__init__(
             self, builtin_name=builtin_name, source_ref=source_ref
@@ -234,7 +244,6 @@ class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
         def createBuiltinMakeException(args, name=None, path=None, source_ref=None):
             if exception_name == "ImportError" and python_version >= 0x300:
                 return ExpressionBuiltinMakeExceptionImportError(
-                    exception_name=exception_name,
                     args=args,
                     name=name,
                     path=path,
@@ -260,3 +269,13 @@ class ExpressionBuiltinExceptionRef(ExpressionBuiltinRefBase):
         assert new_node is not None
 
         return new_node, "new_expression", "Detected built-in exception making."
+
+    def computeExpressionCallViaVariable(
+        self, call_node, variable_ref_node, call_args, call_kw, trace_collection
+    ):
+        return self.computeExpressionCall(
+            call_node=call_node,
+            call_args=call_args,
+            call_kw=call_kw,
+            trace_collection=trace_collection,
+        )
