@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,34 +26,81 @@ from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 
 from .BuiltinTypeNodes import ExpressionBuiltinBool
 from .Checkers import checkStatementsSequenceOrNone
-from .ExpressionBases import ExpressionChildrenHavingBase
-from .NodeBases import StatementChildrenHavingBase
+from .ChildrenHavingMixins import (
+    ChildrenHavingConditionExpressionYesExpressionNoMixin,
+    ChildrenHavingLeftRightMixin,
+)
+from .ExpressionBases import ExpressionBase
 from .NodeMakingHelpers import (
+    makeConstantReplacementNode,
     makeStatementExpressionOnlyReplacementNode,
     wrapExpressionWithNodeSideEffects,
+    wrapExpressionWithSideEffects,
     wrapStatementWithSideEffects,
 )
 from .OperatorNodesUnary import ExpressionOperationNot
-from .shapes.BuiltinTypeShapes import tshape_bool
+from .shapes.BuiltinTypeShapes import tshape_bool, tshape_unknown
+from .StatementBasesGenerated import StatementConditionalBase
 from .StatementNodes import StatementsSequence
 
 
-class ExpressionConditional(ExpressionChildrenHavingBase):
+class ConditionalValueComputeMixin(object):
+    __slots__ = ()
+
+    def _computeConditionTruthValue(self, trace_collection):
+        condition = self.subnode_condition
+
+        truth_value, replacement, message = condition.computeExpressionBool(
+            trace_collection
+        )
+        if replacement is not None:
+            assert replacement.isExpression()
+
+            self.replaceChild(condition, replacement)
+
+            trace_collection.signalChange(
+                tags="new_constant", source_ref=self.source_ref, message=message
+            )
+
+            return truth_value, replacement
+        else:
+            # Query the truth value after the expression is evaluated, once it is
+            # evaluated in onExpression, it is known.
+            return condition.getTruthValue(), condition
+
+
+class ExpressionConditional(
+    ConditionalValueComputeMixin,
+    ChildrenHavingConditionExpressionYesExpressionNoMixin,
+    ExpressionBase,
+):
     kind = "EXPRESSION_CONDITIONAL"
 
     named_children = ("condition", "expression_yes", "expression_no")
 
     def __init__(self, condition, expression_yes, expression_no, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenHavingConditionExpressionYesExpressionNoMixin.__init__(
             self,
-            values={
-                "condition": condition,
-                "expression_yes": expression_yes,
-                "expression_no": expression_no,
-            },
-            source_ref=source_ref,
+            condition=condition,
+            expression_yes=expression_yes,
+            expression_no=expression_no,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
+    def getTypeShape(self):
+        yes_shape = self.subnode_expression_yes.getTypeShape()
+
+        if yes_shape is tshape_unknown:
+            return tshape_unknown
+        else:
+            no_shape = self.subnode_expression_no.getTypeShape()
+
+            if no_shape is yes_shape:
+                return no_shape
+            else:
+                return tshape_unknown
+
     def getBranches(self):
         return (self.subnode_expression_yes, self.subnode_expression_no)
 
@@ -62,11 +109,11 @@ class ExpressionConditional(ExpressionChildrenHavingBase):
 
         # Query the truth value after the expression is evaluated, once it is
         # evaluated in onExpression, it is known.
-        condition = trace_collection.onExpression(expression=self.subnode_condition)
+        condition = trace_collection.onExpression(self.subnode_condition)
 
         # No need to look any further, if the condition raises, the branches do
         # not matter at all.
-        if condition.willRaiseException(BaseException):
+        if condition.willRaiseAnyException():
             return (
                 condition,
                 "new_raise",
@@ -77,11 +124,7 @@ branches.""",
 
         # Tell it we are evaluation it for boolean value only, it may demote
         # itself possibly.
-        condition.computeExpressionBool(trace_collection)
-        condition = self.subnode_condition
-
-        # Decide this based on truth value of condition.
-        truth_value = condition.getTruthValue()
+        truth_value, condition = self._computeConditionTruthValue(trace_collection)
 
         # TODO: We now know that condition evaluates to true for the yes branch
         # and to not true for no branch, the branch should know that.
@@ -94,13 +137,10 @@ branches.""",
                 parent=trace_collection, name="conditional expression yes branch"
             )
 
-            branch_yes_collection.computeBranch(branch=yes_branch)
-
-            # May have just gone away, so fetch it again.
-            yes_branch = self.subnode_expression_yes
+            yes_branch = branch_yes_collection.onExpression(yes_branch)
 
             # If it's aborting, it doesn't contribute to merging.
-            if yes_branch.willRaiseException(BaseException):
+            if truth_value is not True and yes_branch.willRaiseAnyException():
                 branch_yes_collection = None
         else:
             branch_yes_collection = None
@@ -113,13 +153,10 @@ branches.""",
                 parent=trace_collection, name="conditional expression no branch"
             )
 
-            branch_no_collection.computeBranch(branch=no_branch)
-
-            # May have just gone away, so fetch it again.
-            no_branch = self.subnode_expression_no
+            no_branch = branch_no_collection.onExpression(no_branch)
 
             # If it's aborting, it doesn't contribute to merging.
-            if no_branch.willRaiseException(BaseException):
+            if truth_value is not False and no_branch.willRaiseAnyException():
                 branch_no_collection = None
         else:
             branch_no_collection = None
@@ -173,9 +210,7 @@ Convert conditional expression with unused result into conditional statement."""
         )
 
     def mayHaveSideEffectsBool(self):
-        if self.subnode_condition.mayHaveSideEffectsBool():
-            return True
-
+        # The bool will me made on either side.
         if self.subnode_expression_yes.mayHaveSideEffectsBool():
             return True
 
@@ -229,22 +264,22 @@ Convert conditional expression with unused result into conditional statement."""
             return None
 
 
-class ExpressionConditionalBoolBase(ExpressionChildrenHavingBase):
+class ExpressionConditionalBoolBase(ChildrenHavingLeftRightMixin, ExpressionBase):
     named_children = ("left", "right")
 
     def __init__(self, left, right, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self, values={"left": left, "right": right}, source_ref=source_ref
-        )
+        ChildrenHavingLeftRightMixin.__init__(self, left=left, right=right)
+
+        ExpressionBase.__init__(self, source_ref)
 
     def computeExpressionRaw(self, trace_collection):
         # Query the truth value after the expression is evaluated, once it is
         # evaluated in onExpression, it is known.
-        left = trace_collection.onExpression(expression=self.subnode_left)
+        left = trace_collection.onExpression(self.subnode_left)
 
         # No need to look any further, if the condition raises, the branches do
         # not matter at all.
-        if left.willRaiseException(BaseException):
+        if left.willRaiseAnyException():
             return (
                 left,
                 "new_raise",
@@ -277,13 +312,10 @@ branches."""
                 name="boolean %s right branch" % self.conditional_kind,
             )
 
-            branch_yes_collection.computeBranch(branch=right)
-
-            # May have just gone away, so fetch it again.
-            right = self.subnode_right
+            right = branch_yes_collection.onExpression(right)
 
             # If it's aborting, it doesn't contribute to merging.
-            if right.willRaiseException(BaseException):
+            if right.willRaiseAnyException():
                 branch_yes_collection = None
         else:
             branch_yes_collection = None
@@ -325,7 +357,72 @@ branches."""
 
         return False
 
+    def computeExpressionBool(self, trace_collection):
+        truth_value = self.subnode_right.getTruthValue()
+
+        if not self.mayRaiseException(BaseException) and self.mayRaiseExceptionBool(
+            BaseException
+        ):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+        if truth_value is None:
+            # No action
+            pass
+        elif truth_value is True:
+            if self.isExpressionConditionalOr():
+                result = wrapExpressionWithSideEffects(
+                    new_node=makeConstantReplacementNode(
+                        constant=True, node=self.subnode_right, user_provided=False
+                    ),
+                    old_node=self,
+                    side_effects=(
+                        ExpressionBuiltinBool(
+                            value=self.subnode_left, source_ref=self.source_ref
+                        ),
+                    ),
+                )
+
+                return True, result, "Predicted right hand side of 'or' to be true."
+            else:
+                # TODO: There are no side effects after something else, which
+                # would be needed, but for constant values, this works well.
+                if not self.subnode_right.mayHaveSideEffects():
+                    return (
+                        self.subnode_left.getTruthValue(),
+                        self.subnode_left,
+                        "Predicted right hand side of 'and' to be true.",
+                    )
+        elif truth_value is False:
+            if self.isExpressionConditionalAnd():
+                result = wrapExpressionWithSideEffects(
+                    new_node=makeConstantReplacementNode(
+                        constant=False, node=self.subnode_right, user_provided=False
+                    ),
+                    old_node=self,
+                    side_effects=(
+                        ExpressionBuiltinBool(
+                            value=self.subnode_left, source_ref=self.source_ref
+                        ),
+                    ),
+                )
+
+                return False, result, "Predicted right hand side of 'and' to be false."
+            else:
+                # TODO: There are no side effects after something else, which
+                # would be needed, but for constant values, this works well.
+                if not self.subnode_right.mayHaveSideEffects():
+                    return (
+                        self.subnode_left.getTruthValue(),
+                        self.subnode_left,
+                        "Predicted right hand side of 'or' to be false.",
+                    )
+                else:
+                    truth_value = self.subnode_left.getTruthValue()
+
+        return truth_value, None, None
+
     def mayRaiseExceptionBool(self, exception_type):
+        # The and/or bool will be working on either side.
         if self.subnode_left.mayRaiseExceptionBool(exception_type):
             return True
 
@@ -404,27 +501,20 @@ Convert conditional 'and' expression with unused result into conditional stateme
         )
 
 
-class StatementConditional(StatementChildrenHavingBase):
+class StatementConditional(ConditionalValueComputeMixin, StatementConditionalBase):
     kind = "STATEMENT_CONDITIONAL"
 
-    named_children = ("condition", "yes_branch", "no_branch")
+    named_children = (
+        "condition",
+        "yes_branch|statements_or_none+setter",
+        "no_branch|statements_or_none+setter",
+    )
 
     checkers = {
         "yes_branch": checkStatementsSequenceOrNone,
         "no_branch": checkStatementsSequenceOrNone,
     }
 
-    def __init__(self, condition, yes_branch, no_branch, source_ref):
-        StatementChildrenHavingBase.__init__(
-            self,
-            values={
-                "condition": condition,
-                "yes_branch": yes_branch,
-                "no_branch": no_branch,
-            },
-            source_ref=source_ref,
-        )
-
     def isStatementAborting(self):
         yes_branch = self.subnode_yes_branch
 
@@ -490,11 +580,11 @@ class StatementConditional(StatementChildrenHavingBase):
     def computeStatement(self, trace_collection):
         # This is rather complex stuff, pylint: disable=too-many-branches,too-many-statements
 
-        condition = trace_collection.onExpression(expression=self.subnode_condition)
+        condition = trace_collection.onExpression(self.subnode_condition)
 
         # No need to look any further, if the condition raises, the branches do
         # not matter at all.
-        if condition.willRaiseException(BaseException):
+        if condition.willRaiseAnyException():
             result = makeStatementExpressionOnlyReplacementNode(
                 expression=condition, node=self
             )
@@ -509,12 +599,7 @@ branches.""",
 
         # Tell it we are evaluation it for boolean value only, it may demote
         # itself possibly.
-        condition.computeExpressionBool(trace_collection)
-        condition = self.subnode_condition
-
-        # Query the truth value after the expression is evaluated, once it is
-        # evaluated in onExpression, it is known.
-        truth_value = condition.getTruthValue()
+        truth_value, condition = self._computeConditionTruthValue(trace_collection)
 
         # TODO: We now know that condition evaluates to true for the yes branch
         # and to not true for no branch, the branch collection should know that.
@@ -527,14 +612,14 @@ branches.""",
                 yes_branch.finalize()
                 yes_branch = None
 
-                self.clearChild("yes_branch")
+                self.setChildYesBranch(None)
 
         if no_branch is not None:
             if not no_branch.subnode_statements:
                 no_branch.finalize()
                 no_branch = None
 
-                self.clearChild("no_branch")
+                self.setChildNoBranch(None)
 
         # Consider to not remove branches that we know won't be taken.
         if yes_branch is not None and truth_value is False:
@@ -545,9 +630,10 @@ branches.""",
             )
 
             yes_branch.finalize()
-            self.clearChild("yes_branch")
             yes_branch = None
 
+            self.setChildYesBranch(None)
+
         if no_branch is not None and truth_value is True:
             trace_collection.signalChange(
                 tags="new_statements",
@@ -556,9 +642,13 @@ branches.""",
             )
 
             no_branch.finalize()
-            self.clearChild("no_branch")
             no_branch = None
 
+            self.setChildNoBranch(None)
+
+        # Do we need to merge branches
+        needs_merge = True
+
         # Continue to execute for yes branch unless we know it's not going to be
         # relevant.
         if yes_branch is not None:
@@ -566,14 +656,14 @@ branches.""",
                 parent=trace_collection, name="conditional yes branch"
             )
 
-            branch_yes_collection.computeBranch(branch=yes_branch)
-
-            # May have just gone away, so fetch it again.
-            yes_branch = self.subnode_yes_branch
+            yes_branch = branch_yes_collection.computeBranch(branch=yes_branch)
 
             # If it's aborting, it doesn't contribute to merging.
-            if yes_branch is None or yes_branch.isStatementAborting():
+            if yes_branch is None:
                 branch_yes_collection = None
+            elif yes_branch.isStatementAborting():
+                branch_yes_collection = None
+                needs_merge = False
         else:
             branch_yes_collection = None
 
@@ -583,14 +673,14 @@ branches.""",
                 parent=trace_collection, name="conditional no branch"
             )
 
-            branch_no_collection.computeBranch(branch=no_branch)
-
-            # May have just gone away, so fetch it again.
-            no_branch = self.subnode_no_branch
+            no_branch = branch_no_collection.computeBranch(branch=no_branch)
 
             # If it's aborting, it doesn't contribute to merging.
-            if no_branch is None or no_branch.isStatementAborting():
+            if no_branch is None:
                 branch_no_collection = None
+            elif no_branch.isStatementAborting():
+                branch_no_collection = None
+                needs_merge = False
         else:
             branch_no_collection = None
 
@@ -601,7 +691,15 @@ branches.""",
             if branch_no_collection is not None:
                 trace_collection.replaceBranch(branch_no_collection)
         else:
-            trace_collection.mergeBranches(branch_yes_collection, branch_no_collection)
+            if needs_merge:
+                trace_collection.mergeBranches(
+                    branch_yes_collection, branch_no_collection
+                )
+            else:
+                if branch_yes_collection is not None:
+                    trace_collection.replaceBranch(branch_yes_collection)
+                elif branch_no_collection is not None:
+                    trace_collection.replaceBranch(branch_no_collection)
 
         # Both branches may have become empty, which case, the statement needs
         # not remain.
@@ -744,7 +842,7 @@ Empty 'yes' branch for conditional statement treated with inverted condition che
 def makeNotExpression(expression):
     # These are invertible with bool type shape.
     if expression.isExpressionComparison() and expression.getTypeShape() is tshape_bool:
-        return expression.makeInverseComparision()
+        return expression.makeInverseComparison()
     else:
         return ExpressionOperationNot(
             operand=expression, source_ref=expression.getSourceReference()
