@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,17 +21,13 @@ For built-in name references, we check if it's one of the supported built-in
 types, and then specialize for the ones, where it makes sense.
 """
 
-from nuitka.__past__ import xrange  # pylint: disable=I0021,redefined-builtin
+from nuitka.__past__ import unicode, xrange
 from nuitka.Errors import NuitkaAssumptionError
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementDelVariable,
-)
 from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
     ExpressionBuiltinGetattr,
     ExpressionBuiltinHasattr,
     ExpressionBuiltinSetattr,
+    makeExpressionAttributeLookup,
 )
 from nuitka.nodes.BuiltinAllNodes import ExpressionBuiltinAll
 from nuitka.nodes.BuiltinAnyNodes import ExpressionBuiltinAny
@@ -70,7 +66,10 @@ from nuitka.nodes.BuiltinNextNodes import (
     ExpressionBuiltinNext1,
     ExpressionBuiltinNext2,
 )
-from nuitka.nodes.BuiltinOpenNodes import ExpressionBuiltinOpen
+from nuitka.nodes.BuiltinOpenNodes import (
+    ExpressionBuiltinOpenP2,
+    ExpressionBuiltinOpenP3,
+)
 from nuitka.nodes.BuiltinRangeNodes import (
     ExpressionBuiltinRange1,
     ExpressionBuiltinRange2,
@@ -119,12 +118,14 @@ from nuitka.nodes.GlobalsLocalsNodes import (
     ExpressionBuiltinGlobals,
 )
 from nuitka.nodes.ImportNodes import ExpressionBuiltinImport
+from nuitka.nodes.KeyValuePairNodes import (
+    makeKeyValuePairExpressionsFromKwArgs,
+)
 from nuitka.nodes.NodeMakingHelpers import (
     makeConstantReplacementNode,
     makeExpressionBuiltinLocals,
     makeRaiseExceptionReplacementExpression,
     makeRaiseExceptionReplacementExpressionFromInstance,
-    wrapExpressionWithSideEffects,
 )
 from nuitka.nodes.OperatorNodes import ExpressionOperationBinaryDivmod
 from nuitka.nodes.OperatorNodesUnary import (
@@ -139,16 +140,20 @@ from nuitka.nodes.TypeNodes import (
     ExpressionBuiltinIsinstance,
     ExpressionBuiltinIssubclass,
     ExpressionBuiltinSuper0,
+    ExpressionBuiltinSuper1,
     ExpressionBuiltinSuper2,
     ExpressionBuiltinType1,
 )
+from nuitka.nodes.VariableAssignNodes import (
+    makeStatementAssignmentVariable,
+    makeStatementDelVariable,
+)
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
 )
 from nuitka.PythonVersions import python_version
 from nuitka.specs import BuiltinParameterSpecs
-from nuitka.Tracing import optimization_logger
 from nuitka.tree.ReformulationExecStatements import wrapEvalGlobalsAndLocals
 from nuitka.tree.ReformulationTryFinallyStatements import (
     makeTryFinallyStatement,
@@ -169,7 +174,7 @@ def dir_extractor(node):
         )
 
         result = makeCallNode(
-            ExpressionAttributeLookup(
+            makeExpressionAttributeLookup(
                 expression=source, attribute_name="keys", source_ref=source_ref
             ),
             source_ref,
@@ -310,29 +315,15 @@ def dict_extractor(node):
     # The "dict" built-in is a bit strange in that it accepts a position
     # parameter, or not, but won't have a default value.
     def wrapExpressionBuiltinDictCreation(positional_args, dict_star_arg, source_ref):
-        if len(positional_args) > 1:
-
-            result = makeRaiseExceptionReplacementExpressionFromInstance(
-                expression=node,
-                exception=TypeError(
-                    "dict expected at most 1 arguments, got %d" % (len(positional_args))
-                ),
-            )
-
-            result = wrapExpressionWithSideEffects(
-                side_effects=positional_args, old_node=node, new_node=result
-            )
-
-            if dict_star_arg:
-                result = wrapExpressionWithSideEffects(
-                    side_effects=dict_star_arg, old_node=node, new_node=result
-                )
-
-            return result
+        if positional_args:
+            # Only one allowed, the spec converted too many into an exception.
+            (pos_arg,) = positional_args
+        else:
+            pos_arg = None
 
         return ExpressionBuiltinDict(
-            pos_arg=positional_args[0] if positional_args else None,
-            pairs=dict_star_arg,
+            pos_arg=pos_arg,
+            pairs=makeKeyValuePairExpressionsFromKwArgs(dict_star_arg),
             source_ref=source_ref,
         )
 
@@ -529,34 +520,62 @@ def any_extractor(node):
 
 
 def tuple_extractor(node):
+    def makeTuple0(source_ref):
+        # pylint: disable=unused-argument
+
+        return makeConstantReplacementNode(constant=(), node=node, user_provided=False)
+
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
         builtin_class=ExpressionBuiltinTuple,
         builtin_spec=BuiltinParameterSpecs.builtin_tuple_spec,
+        empty_special_class=makeTuple0,
     )
 
 
 def list_extractor(node):
+    def makeList0(source_ref):
+        # pylint: disable=unused-argument
+
+        return makeConstantReplacementNode(constant=[], node=node, user_provided=False)
+
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
         builtin_class=ExpressionBuiltinList,
         builtin_spec=BuiltinParameterSpecs.builtin_list_spec,
+        empty_special_class=makeList0,
     )
 
 
 def set_extractor(node):
+    def makeSet0(source_ref):
+        # pylint: disable=unused-argument
+
+        return makeConstantReplacementNode(
+            constant=set(), node=node, user_provided=False
+        )
+
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
         builtin_class=ExpressionBuiltinSet,
         builtin_spec=BuiltinParameterSpecs.builtin_set_spec,
+        empty_special_class=makeSet0,
     )
 
 
 def frozenset_extractor(node):
+    def makeFrozenset0(source_ref):
+        # pylint: disable=unused-argument
+
+        return makeConstantReplacementNode(
+            constant=frozenset(), node=node, user_provided=False
+        )
+
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
         builtin_class=ExpressionBuiltinFrozenset,
         builtin_spec=BuiltinParameterSpecs.builtin_frozenset_spec,
+        empty_special_class=makeFrozenset0,
     )
 
 
@@ -601,25 +620,40 @@ def complex_extractor(node):
 
 
 def str_extractor(node):
+    def makeStr0(source_ref):
+        # pylint: disable=unused-argument
+
+        return makeConstantReplacementNode(
+            constant=str(), node=node, user_provided=False
+        )
+
     builtin_class = ExpressionBuiltinStrP2 if str is bytes else ExpressionBuiltinStrP3
 
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
         builtin_class=builtin_class,
         builtin_spec=builtin_class.builtin_spec,
+        empty_special_class=makeStr0,
     )
 
 
 if python_version < 0x300:
 
     def unicode_extractor(node):
+        def makeUnicode0(source_ref):
+            # pylint: disable=unused-argument
+
+            return makeConstantReplacementNode(
+                constant=unicode(), node=node, user_provided=False
+            )
+
         return BuiltinParameterSpecs.extractBuiltinArgs(
             node=node,
             builtin_class=ExpressionBuiltinUnicodeP2,
             builtin_spec=ExpressionBuiltinUnicodeP2.builtin_spec,
+            empty_special_class=makeUnicode0,
         )
 
-
 else:
     from nuitka.nodes.BuiltinTypeNodes import (
         ExpressionBuiltinBytes1,
@@ -771,9 +805,10 @@ if python_version < 0x300:
                     tried,
                     makeStatementReturn(
                         expression=ExpressionBuiltinExecfile(
+                            in_class_body=node.getParentVariableProvider().isExpressionClassBody(),
                             source_code=makeCallNode(
-                                ExpressionAttributeLookup(
-                                    expression=ExpressionBuiltinOpen(
+                                makeExpressionAttributeLookup(
+                                    expression=ExpressionBuiltinOpenP2(
                                         filename=filename,
                                         mode=makeConstantRefNode(
                                             constant="rU", source_ref=source_ref
@@ -797,8 +832,7 @@ if python_version < 0x300:
                 source_ref=source_ref,
             )
 
-            outline_body.setChild(
-                "body",
+            outline_body.setChildBody(
                 makeStatementsSequenceFromStatement(
                     statement=makeTryFinallyStatement(
                         provider=outline_body,
@@ -806,7 +840,7 @@ if python_version < 0x300:
                         final=final,
                         source_ref=source_ref,
                     )
-                ),
+                )
             )
 
             return outline_body
@@ -850,14 +884,13 @@ def eval_extractor(node):
             temp_scope=None, name="source"
         )
 
-        final.setChild(
-            "statements",
+        final.setChildStatements(
             final.subnode_statements
             + (
-                StatementDelVariable(
+                makeStatementDelVariable(
                     variable=source_variable, tolerant=True, source_ref=source_ref
                 ),
-            ),
+            )
         )
 
         strip_choice = makeConstantRefNode(constant=(" \t",), source_ref=source_ref)
@@ -885,10 +918,10 @@ def eval_extractor(node):
 
         # Source needs some special treatment for eval, if it's a string, it
         # must be stripped.
-        string_fixup = StatementAssignmentVariable(
+        string_fixup = makeStatementAssignmentVariable(
             variable=source_variable,
             source=makeExpressionCall(
-                called=ExpressionAttributeLookup(
+                called=makeExpressionAttributeLookup(
                     expression=ExpressionTempVariableRef(
                         variable=source_variable, source_ref=source_ref
                     ),
@@ -914,7 +947,7 @@ def eval_extractor(node):
             )
 
         statements = (
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=source_variable, source=source, source_ref=source_ref
             ),
             makeStatementConditional(
@@ -924,7 +957,7 @@ def eval_extractor(node):
                             variable=source_variable, source_ref=source_ref
                         ),
                         classes=makeExpressionMakeTupleOrConstant(
-                            elements=acceptable_builtin_types,
+                            elements=tuple(acceptable_builtin_types),
                             user_provided=True,
                             source_ref=source_ref,
                         ),
@@ -953,8 +986,7 @@ def eval_extractor(node):
             statements=(tried,) + statements, allow_none=False, source_ref=source_ref
         )
 
-        outline_body.setChild(
-            "body",
+        outline_body.setChildBody(
             makeStatementsSequenceFromStatement(
                 statement=makeTryFinallyStatement(
                     provider=outline_body,
@@ -962,7 +994,7 @@ def eval_extractor(node):
                     final=final,
                     source_ref=source_ref,
                 )
-            ),
+            )
         )
 
         return outline_body
@@ -1000,6 +1032,7 @@ if python_version >= 0x300:
                     tried,
                     makeStatementReturn(
                         expression=ExpressionBuiltinExec(
+                            in_early_closure=node.getParentVariableProvider().isEarlyClosure(),
                             source_code=source,
                             globals_arg=globals_ref,
                             locals_arg=locals_ref,
@@ -1015,8 +1048,7 @@ if python_version >= 0x300:
             # Hack: Allow some APIs to work already
             tried.parent = outline_body
 
-            outline_body.setChild(
-                "body",
+            outline_body.setChildBody(
                 makeStatementsSequenceFromStatement(
                     statement=makeTryFinallyStatement(
                         provider=provider,
@@ -1024,7 +1056,7 @@ if python_version >= 0x300:
                         final=final,
                         source_ref=source_ref,
                     )
-                ),
+                )
             )
 
             return outline_body
@@ -1061,6 +1093,8 @@ def open_extractor(node):
     def makeOpen0(source_ref):
         # pylint: disable=unused-argument
         try:
+            # Not giving arguments or context on purpose
+            # pylint: disable=consider-using-with,unspecified-encoding
             open()
         except Exception as e:  # We want to broad here, pylint: disable=broad-except
             return makeRaiseExceptionReplacementExpressionFromInstance(
@@ -1071,7 +1105,9 @@ def open_extractor(node):
 
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
-        builtin_class=ExpressionBuiltinOpen,
+        builtin_class=ExpressionBuiltinOpenP3
+        if str is not bytes
+        else ExpressionBuiltinOpenP2,
         builtin_spec=BuiltinParameterSpecs.builtin_open_spec,
         empty_special_class=makeOpen0,
     )
@@ -1079,6 +1115,8 @@ def open_extractor(node):
 
 def super_extractor(node):
     def wrapSuperBuiltin(type_arg, object_arg, source_ref):
+        # Many cases due to complex defaulting behavior, pylint: disable=too-many-return-statements
+
         if type_arg is None and python_version >= 0x300:
             if provider.isCompiledPythonModule():
                 return makeRaiseExceptionReplacementExpression(
@@ -1154,12 +1192,22 @@ def super_extractor(node):
                         )
 
             return ExpressionBuiltinSuper0(
-                type_arg=type_arg, object_arg=object_arg, source_ref=source_ref
+                type_arg=type_arg,
+                object_arg=object_arg,
+                source_ref=source_ref,
             )
 
-        return ExpressionBuiltinSuper2(
-            type_arg=type_arg, object_arg=object_arg, source_ref=source_ref
-        )
+        if object_arg is None:
+            return ExpressionBuiltinSuper1(
+                type_arg=type_arg,
+                source_ref=source_ref,
+            )
+        else:
+            return ExpressionBuiltinSuper2(
+                type_arg=type_arg,
+                object_arg=object_arg,
+                source_ref=source_ref,
+            )
 
     provider = node.getParentVariableProvider().getEntryPoint()
 
@@ -1235,7 +1283,8 @@ def bytearray_extractor(node):
     def makeBytearray0(source_ref):
         return makeConstantRefNode(constant=bytearray(), source_ref=source_ref)
 
-    def selectNextBuiltinClass(string, encoding, errors, source_ref):
+    def selectBytearrayBuiltinClass(string, encoding, errors, source_ref):
+        # TODO: Highly suspicious that "errors"
         if encoding is None:
             return ExpressionBuiltinBytearray1(value=string, source_ref=source_ref)
         else:
@@ -1245,7 +1294,7 @@ def bytearray_extractor(node):
 
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
-        builtin_class=selectNextBuiltinClass,
+        builtin_class=selectBytearrayBuiltinClass,
         builtin_spec=BuiltinParameterSpecs.builtin_bytearray_spec,
         empty_special_class=makeBytearray0,
     )
@@ -1510,9 +1559,10 @@ def computeBuiltinCall(builtin_name, call_node):
 
         return new_node, tags, message
     else:
-        if False and builtin_name not in _builtin_ignore_list:
-            optimization_logger.warning(
-                "Not handling built-in %r, consider support." % builtin_name
-            )
+        # TODO: Achieve coverage of all built-ins in at least the ignore list.
+        # if False and builtin_name not in _builtin_ignore_list:
+        #     optimization_logger.warning(
+        #         "Not handling built-in %r, consider support." % builtin_name
+        #     )
 
         return call_node, None, None
