@@ -33,6 +33,8 @@ from nuitka.utils.Yaml import getYamlPackageConfiguration
 
 # spell-checker: ignore dask,numba
 
+_mode_choices = ("error", "warning", "nofollow", "allow")
+
 
 class NuitkaPluginAntiBloat(NuitkaPluginBase):
     plugin_name = "anti-bloat"
@@ -56,7 +58,7 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
         custom_choices,
         show_changes,
     ):
-        # Many details, due to many repetitive arguments, pylint: disable=too-many-branches
+        # Many details, due to many repetitive arguments, pylint: disable=too-many-branches,too-many-statements
 
         self.show_changes = show_changes
 
@@ -111,6 +113,7 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
 
         if noinclude_numba_mode != "allow":
             self.handled_modules["numba"] = noinclude_numba_mode
+            self.handled_modules["sparse"] = noinclude_numba_mode
         else:
             self.control_tags["use_numba"] = True
 
@@ -123,7 +126,7 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
 
             module_name, mode = custom_choice.rsplit(":", 1)
 
-            if mode not in ("error", "warning", "nofollow", "allow", "bytecode"):
+            if mode not in _mode_choices and mode != "bytecode":
                 self.sysexit(
                     "Error, illegal mode given '%s' in '--noinclude-custom-mode=%s'"
                     % (mode, custom_choice)
@@ -152,7 +155,7 @@ Annotate what changes are by the plugin done.""",
             "--noinclude-setuptools-mode",
             action="store",
             dest="noinclude_setuptools_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default=None,
             help="""\
 What to do if a 'setuptools' or import is encountered. This package can be big with
@@ -163,7 +166,7 @@ dependencies, and should definitely be avoided. Also handles 'setuptools_scm'.""
             "--noinclude-pytest-mode",
             action="store",
             dest="noinclude_pytest_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default=None,
             help="""\
 What to do if a 'pytest' import is encountered. This package can be big with
@@ -174,7 +177,7 @@ dependencies, and should definitely be avoided. Also handles 'nose' imports.""",
             "--noinclude-unittest-mode",
             action="store",
             dest="noinclude_unittest_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default=None,
             help="""\
 What to do if a unittest import is encountered. This package can be big with
@@ -185,7 +188,7 @@ dependencies, and should definitely be avoided.""",
             "--noinclude-IPython-mode",
             action="store",
             dest="noinclude_ipython_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default=None,
             help="""\
 What to do if a IPython import is encountered. This package can be big with
@@ -196,7 +199,7 @@ dependencies, and should definitely be avoided.""",
             "--noinclude-dask-mode",
             action="store",
             dest="noinclude_dask_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default=None,
             help="""\
 What to do if a 'dask' import is encountered. This package can be big with
@@ -207,7 +210,7 @@ dependencies, and should definitely be avoided.""",
             "--noinclude-numba-mode",
             action="store",
             dest="noinclude_numba_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default=None,
             help="""\
 What to do if a 'numba' import is encountered. This package can be big with
@@ -219,7 +222,7 @@ big with dependencies, and should definitely be avoided.""",
             "--noinclude-default-mode",
             action="store",
             dest="noinclude_default_mode",
-            choices=("error", "warning", "nofollow", "allow"),
+            choices=_mode_choices,
             default="warning",
             help="""\
 This actually provides the default "warning" value for above options, and
@@ -249,6 +252,7 @@ which can and should be a top level package and then one choice, "error",
         context_code = anti_bloat_config.get("context", "")
         if type(context_code) in (tuple, list):
             context_code = "\n".join(context_code)
+        context["version"] = self.getPackageVersion
 
         # We trust the yaml files, pylint: disable=eval-used,exec-used
         context_ready = not bool(context_code)
@@ -440,14 +444,23 @@ which can and should be a top level package and then one choice, "error",
                     body=body,
                 )
 
-    def onModuleRecursion(self, module_name, module_filename, module_kind):
+    def onModuleRecursion(
+        self, module_name, module_filename, module_kind, using_module, source_ref
+    ):
         for handled_module_name, mode in self.handled_modules.items():
             if module_name.hasNamespace(handled_module_name):
-                # Make sure the compilation aborts.
+                # Make sure the compilation aborts or warns if asked to
                 if mode == "error":
                     raise NuitkaForbiddenImportEncounter(module_name)
-                if mode == "warning":
-                    self.warning("Unwanted import of '%s' encountered." % module_name)
+                if mode == "warning" and (
+                    using_module is None
+                    or not using_module.getFullName().hasNamespace(handled_module_name)
+                ):
+                    self.warning(
+                        "Undesirable import of '%s' at '%s' encountered. It may slow down compilation."
+                        % (handled_module_name, source_ref.getAsString()),
+                        mnemonic="unwanted-module",
+                    )
 
     def onModuleEncounter(self, module_name, module_filename, module_kind):
         for handled_module_name, mode in self.handled_modules.items():
