@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,10 +25,10 @@ import hashlib
 import os
 import pickle
 import re
-import subprocess
 import sys
 import time
 
+from nuitka.OptionParsing import getNuitkaProjectOptions
 from nuitka.PythonVersions import python_version
 from nuitka.tools.testing.Common import (
     addToPythonPath,
@@ -36,7 +36,8 @@ from nuitka.tools.testing.Common import (
     getDebugPython,
     getTempDir,
     getTestingCPythonOutputsCacheDir,
-    killProcess,
+    killProcessGroup,
+    test_logger,
     withPythonPathChange,
 )
 from nuitka.tools.testing.OutputComparison import compareOutput
@@ -44,6 +45,7 @@ from nuitka.Tracing import my_print
 from nuitka.utils.Execution import (
     callProcess,
     check_output,
+    createProcess,
     executeProcess,
     wrapCommandForDebuggerForSubprocess,
 )
@@ -71,7 +73,7 @@ def checkNoPermissionError(output):
         if candidate in output:
             return False
 
-    # These are localized it seems.
+    # These are localized it seems, spell-checker: ignore totest
     if re.search(
         b"(WindowsError|FileNotFoundError|FileExistsError|WinError 145):"
         b".*(@test|totest|xx|Error 145)",
@@ -95,15 +97,14 @@ def _getCPythonResults(cpython_cmd, send_kill):
         stop_watch.start()
 
         with withPythonPathChange(os.getcwd()):
-            # want fine grained control, pylint: disable=consider-using-with
-            process = subprocess.Popen(
-                args=cpython_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-            )
+            process = createProcess(command=cpython_cmd, new_group=send_kill)
 
         if send_kill:
             # Doing it per loop iteration hopefully, pylint: disable=cell-var-from-loop
             executeAfterTimePassed(
-                1.0, lambda: killProcess("Uncompiled Python program", process.pid)
+                message="Scheduling process kill in %.02fs",
+                timeout=2.0,
+                func=lambda: killProcessGroup("Uncompiled Python program", process.pid),
             )
 
         stdout_cpython, stderr_cpython = process.communicate()
@@ -149,7 +150,7 @@ def getCPythonResults(cpython_cmd, cpython_cached, force_update, send_kill):
         command_hash.update(hash_salt)
 
         if os.name == "nt" and python_version < 0x300:
-            curdir = os.getcwdu()
+            curdir = os.getcwdu()  # spell-checker: ignore getcwdu
         else:
             curdir = os.getcwd()
 
@@ -254,9 +255,9 @@ def main():
     full_compat = not hasArg("improved")
     cpython_cached = hasArg("cpython_cache")
     syntax_errors = hasArg("syntax_errors")
-    noprefer_source = hasArg("noprefer_source")
-    noverbose_log = hasArg("noverbose_log")
-    noinclusion_log = hasArg("noinclusion_log")
+    no_prefer_source = hasArg("--no-prefer-source")
+    no_verbose_log = hasArg("no_verbose_log")
+    no_inclusion_log = hasArg("no_inclusion_log")
     send_kill = hasArg("--send-ctrl-c")
     output_dir = hasArgValue("--output-dir", None)
     include_packages = hasArgValues("--include-package")
@@ -281,23 +282,25 @@ def main():
             user_plugins.append(arg[len("user_plugin:") :])
             del args[count]
 
-    recurse_not = []
+    nowarn_mnemonics = []
 
     for count, arg in reversed(tuple(enumerate(args))):
-        if arg.startswith("recurse_not:"):
-            recurse_not.append(arg[len("recurse_not:") :])
-            del args[count]
-
-    recurse_to = []
-
-    for count, arg in reversed(tuple(enumerate(args))):
-        if arg.startswith("recurse_to:"):
-            recurse_to.append(arg[len("recurse_to:") :])
+        if arg.startswith("--nowarn-mnemonic="):
+            nowarn_mnemonics.append(arg[len("--nowarn-mnemonic=") :])
             del args[count]
 
     if args:
         sys.exit("Error, non understood mode(s) '%s'," % ",".join(args))
 
+    project_options = tuple(
+        getNuitkaProjectOptions(
+            logger=test_logger, filename_arg=filename, module_mode=module_mode
+        )
+    )
+
+    if "--standalone" in project_options:
+        standalone_mode = True
+
     # In coverage mode, we don't want to execute, and to do this only in one mode,
     # we enable two step execution, which splits running the binary from the actual
     # compilation:
@@ -417,6 +420,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         else:
             assert coverage_mode
 
+            # spell-checker: ignore rcfile
+
             nuitka_call = [
                 os.environ["PYTHON"],
                 "-S",
@@ -448,7 +453,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if full_compat:
         extra_options.append("--full-compat")
 
-    if noprefer_source:
+    if no_prefer_source:
         extra_options.append("--no-prefer-source")
 
     if python_flag_m:
@@ -479,14 +484,14 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     if report:
         extra_options.append("--report=%s" % report)
 
-    if nofollow_imports:
+    if nofollow_imports or (not follow_imports and not standalone_mode):
         extra_options.append("--nofollow-imports")
 
     if follow_imports:
         extra_options.append("--follow-imports")
 
-    if recurse_not:
-        extra_options.extend("--nofollow-import-to=" + v for v in recurse_not)
+    if nowarn_mnemonics:
+        extra_options.extend("--nowarn-mnemonic=" + v for v in nowarn_mnemonics)
 
     if coverage_mode:
         extra_options.append("--must-not-re-execute")
@@ -501,10 +506,10 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
     for user_plugin in user_plugins:
         extra_options.append("--user-plugin=" + user_plugin)
 
-    if not noverbose_log:
+    if not no_verbose_log:
         extra_options.append("--verbose-output=%s.optimization.log" % filename)
 
-    if not noinclusion_log:
+    if not no_inclusion_log:
         extra_options.append("--show-modules-output=%s.inclusion.log" % filename)
 
     if output_dir is not None:
@@ -621,9 +626,7 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
         # be transient.
         for _i in range(5):
             with withPythonPathChange(nuitka_package_dir):
-                process = subprocess.Popen(
-                    args=nuitka_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-                )
+                process = createProcess(command=nuitka_cmd)
 
             stdout_nuitka, stderr_nuitka = process.communicate()
             exit_nuitka = process.returncode
@@ -646,6 +649,8 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
                     nuitka_cmd1
                 )
 
+                python_path_used = os.environ["PYTHONPATH"]
+
             if exit_nuitka1 != 0:
                 if (
                     not expect_failure
@@ -654,12 +659,12 @@ Taking coverage of '{filename}' using '{python}' with flags {args} ...""".format
                 ):
                     sys.exit(
                         """\
-Error, failed to take coverage with '%s'.
+Error, failed to take coverage with '%s' (PYTHONPATH was '%s').
 
 Stderr was:
 %s
 """
-                        % (os.environ["PYTHON"], stderr_nuitka1)
+                        % (nuitka_cmd1, python_path_used, stderr_nuitka1)
                     )
 
                 exit_nuitka = exit_nuitka1
@@ -675,17 +680,17 @@ Stderr was:
                     if trace_command:
                         my_print("Nuitka command 2:", nuitka_cmd2)
 
-                    # Need full manual control, and not all Python versions allow using
-                    # context manager here, pylint: disable=consider-using-with
-                    process = subprocess.Popen(
-                        args=nuitka_cmd2, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-                    )
+                    # Need full manual control
+                    process = createProcess(command=nuitka_cmd2, new_group=send_kill)
 
                     if send_kill:
                         # Lambda is used immediately in same loop, pylint: disable=cell-var-from-loop
                         executeAfterTimePassed(
-                            1.0,
-                            lambda: killProcess("Nuitka compiled program", process.pid),
+                            message="Scheduling process kill in %.02fs",
+                            timeout=2.0,
+                            func=lambda: killProcessGroup(
+                                "Nuitka compiled program", process.pid
+                            ),
                         )
 
                     stdout_nuitka2, stderr_nuitka2 = process.communicate()
