@@ -157,25 +157,21 @@ def _buildPython2ListContraction(provider, node, source_ref):
         source_ref=source_ref,
     )
 
-    statements.append(
-        StatementReturn(
-            expression=ExpressionTempVariableRef(
-                variable=container_tmp, source_ref=source_ref
-            ),
-            source_ref=source_ref,
-        )
+    return_statement = StatementReturn(
+        expression=ExpressionTempVariableRef(
+            variable=container_tmp, source_ref=source_ref
+        ),
+        source_ref=source_ref,
     )
 
     statement = makeTryFinallyStatement(
         provider=function_body,
-        tried=statements,
+        tried=mergeStatements((statements, return_statement)),
         final=release_statements,
         source_ref=source_ref.atInternal(),
     )
 
-    function_body.setChild(
-        "body", makeStatementsSequenceFromStatement(statement=statement)
-    )
+    function_body.setChildBody(makeStatementsSequenceFromStatement(statement=statement))
 
     return function_body
 
@@ -240,6 +236,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
     code_object = CodeObjectSpec(
         co_name="<genexpr>",
+        co_qualname=provider.getChildQualname("<genexpr>"),
         co_kind="Generator",
         co_varnames=(".0",),
         co_freevars=(),
@@ -285,8 +282,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
         maker_class = ExpressionMakeGeneratorObject
 
-    function_body.setChild(
-        "body",
+    function_body.setChildBody(
         makeStatementsSequenceFromStatements(
             makeStatementAssignmentVariable(
                 variable=iter_tmp,
@@ -314,7 +310,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
                 ),
                 source_ref=source_ref,
             ),
-        ),
+        )
     )
 
     statements, release_statements = _buildContractionBodyNode(
@@ -332,7 +328,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
     )
 
     if is_async:
-        statements.append(StatementGeneratorReturnNone(source_ref=source_ref))
+        statements += (StatementGeneratorReturnNone(source_ref=source_ref),)
 
     statements = (
         makeTryFinallyStatement(
@@ -343,15 +339,14 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
         ),
     )
 
-    code_body.setChild(
-        "body",
+    code_body.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=StatementsFrameGenerator(
                 statements=mergeStatements(statements, False),
                 code_object=code_object,
                 source_ref=source_ref,
             )
-        ),
+        )
     )
 
     return function_body
@@ -579,10 +574,10 @@ def _buildContractionBodyNode(
     statements.append(current_body)
     statements = mergeStatements(statements)
 
-    release_statements = [
+    release_statements = tuple(
         makeStatementReleaseVariable(variable=tmp_variable, source_ref=source_ref)
         for tmp_variable in tmp_variables
-    ]
+    )
 
     return statements, release_statements
 
@@ -590,7 +585,7 @@ def _buildContractionBodyNode(
 def _buildContractionNode(provider, node, name, emit_class, start_value, source_ref):
     # The contraction nodes are reformulated to function bodies, with loops as
     # described in the Developer Manual. They use a lot of temporary names,
-    # nested blocks, etc. and so a lot of variable names.
+    # nested blocks, etc. and so a lot of variable names, pylint:disable=too-many-locals
 
     function_body = ExpressionOutlineFunction(
         provider=provider, name=intern(name[1:-1]), source_ref=source_ref
@@ -627,19 +622,17 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
         source_ref=source_ref,
     )
 
-    statements.append(
-        StatementReturn(
-            expression=ExpressionTempVariableRef(
-                variable=container_tmp, source_ref=source_ref
-            ),
-            source_ref=source_ref,
-        )
+    return_statement = StatementReturn(
+        expression=ExpressionTempVariableRef(
+            variable=container_tmp, source_ref=source_ref
+        ),
+        source_ref=source_ref,
     )
 
     statements = (
         makeTryFinallyStatement(
             provider=function_body,
-            tried=statements,
+            tried=mergeStatements((statements, return_statement)),
             final=release_statements,
             source_ref=source_ref.atInternal(),
         ),
@@ -652,6 +645,7 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
 
         code_object = CodeObjectSpec(
             co_name=name,
+            co_qualname=provider.getChildQualname(name),
             co_kind="Function",
             co_varnames=(),
             co_freevars=(),
@@ -674,6 +668,6 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
             ),
         )
 
-    function_body.setChild("body", body)
+    function_body.setChildBody(body)
 
     return function_body
