@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,11 +22,6 @@ source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementAssignmentVariableName,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AsyncgenNodes import (
     ExpressionAsyncgenObjectBody,
     ExpressionMakeAsyncgenObject,
@@ -44,8 +39,8 @@ from nuitka.nodes.CoroutineNodes import (
 )
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionBody,
-    ExpressionFunctionCreation,
     ExpressionFunctionRef,
+    makeExpressionFunctionCreation,
 )
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
@@ -55,11 +50,16 @@ from nuitka.nodes.GeneratorNodes import (
 from nuitka.nodes.LocalsDictNodes import StatementSetLocalsDictionary
 from nuitka.nodes.OutlineNodes import ExpressionOutlineFunction
 from nuitka.nodes.ReturnNodes import StatementReturn, StatementReturnNone
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableNameNodes import (
+    ExpressionVariableNameRef,
+    StatementAssignmentVariableName,
+)
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
-    ExpressionVariableNameRef,
     ExpressionVariableRef,
 )
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.Options import hasPythonFlagNoAnnotations
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
@@ -72,6 +72,7 @@ from .TreeHelpers import (
     buildFrameNode,
     buildNode,
     buildNodeList,
+    buildNodeTuple,
     detectFunctionBodyKind,
     extractDocFromBody,
     getKind,
@@ -88,9 +89,8 @@ def _insertFinalReturnStatement(function_statements_body, return_statement):
             statement=return_statement
         )
     elif not function_statements_body.isStatementAborting():
-        function_statements_body.setChild(
-            "statements",
-            function_statements_body.subnode_statements + (return_statement,),
+        function_statements_body.setChildStatements(
+            function_statements_body.subnode_statements + (return_statement,)
         )
 
     return function_statements_body
@@ -106,15 +106,15 @@ def _insertInitialSetLocalsDictStatement(function_body, function_statements_body
             statement=locals_statement
         )
     else:
-        function_statements_body.setChild(
-            "statements",
-            (locals_statement,) + function_statements_body.subnode_statements,
+        function_statements_body.setChildStatements(
+            (locals_statement,) + function_statements_body.subnode_statements
         )
 
     return function_statements_body
 
 
 def _injectDecorator(decorators, inject, acceptable, source_ref):
+    assert type(decorators) is list
     assert type(inject) is str
     assert type(acceptable) is tuple
 
@@ -182,8 +182,7 @@ def buildFunctionNode(provider, node, source_ref):
         for variable in function_body.getProvidedVariables():
             code_body.getVariableForReference(variable.getName())
 
-        function_body.setChild(
-            "body",
+        function_body.setChildBody(
             makeStatementsSequenceFromStatement(
                 statement=StatementReturn(
                     expression=maker_class(
@@ -194,14 +193,14 @@ def buildFunctionNode(provider, node, source_ref):
                     ),
                     source_ref=source_ref,
                 )
-            ),
+            )
         )
 
     decorators = buildNodeList(
         provider=provider, nodes=reversed(node.decorator_list), source_ref=source_ref
     )
 
-    defaults = buildNodeList(
+    defaults = buildNodeTuple(
         provider=provider, nodes=node.args.defaults, source_ref=source_ref
     )
 
@@ -233,11 +232,11 @@ def buildFunctionNode(provider, node, source_ref):
             statement=function_statements_body
         )
 
-    code_body.setChild("body", function_statements_body)
+    code_body.setChildBody(function_statements_body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
 
-    function_creation = ExpressionFunctionCreation(
+    function_creation = makeExpressionFunctionCreation(
         function_ref=ExpressionFunctionRef(
             function_body=function_body, source_ref=source_ref
         ),
@@ -347,7 +346,7 @@ def buildAsyncFunctionNode(provider, node, source_ref):
         provider=provider, nodes=reversed(node.decorator_list), source_ref=source_ref
     )
 
-    defaults = buildNodeList(
+    defaults = buildNodeTuple(
         provider=provider, nodes=node.args.defaults, source_ref=source_ref
     )
 
@@ -368,7 +367,7 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             statement=function_statements_body
         )
 
-    function_body.setChild("body", function_statements_body)
+    function_body.setChildBody(function_statements_body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
 
@@ -394,14 +393,13 @@ def buildAsyncFunctionNode(provider, node, source_ref):
             source_ref=source_ref,
         )
 
-    creator_function_body.setChild(
-        "body",
+    creator_function_body.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=StatementReturn(expression=creation_node, source_ref=source_ref)
-        ),
+        )
     )
 
-    function_creation = ExpressionFunctionCreation(
+    function_creation = makeExpressionFunctionCreation(
         function_ref=ExpressionFunctionRef(
             function_body=creator_function_body, source_ref=source_ref
         ),
@@ -557,7 +555,7 @@ def _wrapFunctionWithSpecialNestedArgs(
         iter_vars.append(iter_var)
 
         statements.append(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=iter_var,
                 source=ExpressionBuiltinIter1(value=source, source_ref=source_ref),
                 source_ref=source_ref,
@@ -570,7 +568,7 @@ def _wrapFunctionWithSpecialNestedArgs(
                 outer_body.getLocalsScope().registerProvidedVariable(arg_var)
 
                 statements.append(
-                    StatementAssignmentVariable(
+                    makeStatementAssignmentVariable(
                         variable=arg_var,
                         source=ExpressionSpecialUnpack(
                             value=ExpressionTempVariableRef(
@@ -640,14 +638,15 @@ def _wrapFunctionWithSpecialNestedArgs(
 
     statements.append(StatementReturn(expression=code_body, source_ref=source_ref))
 
-    outer_body.setChild(
-        "body",
+    outer_body.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=makeTryFinallyStatement(
                 provider=outer_body,
                 tried=statements,
                 final=[
-                    StatementReleaseVariable(variable=variable, source_ref=source_ref)
+                    makeStatementReleaseVariable(
+                        variable=variable, source_ref=source_ref
+                    )
                     for variable in sorted(
                         outer_body.getTempVariables(),
                         key=lambda variable: variable.getName(),
@@ -656,7 +655,7 @@ def _wrapFunctionWithSpecialNestedArgs(
                 source_ref=source_ref,
                 public_exc=False,
             )
-        ),
+        )
     )
 
     return code_body
@@ -727,6 +726,7 @@ def buildFunctionWithParsing(
 
     code_object = CodeObjectSpec(
         co_name=name,
+        co_qualname=provider.getChildQualname(name),
         co_kind=function_kind,
         co_varnames=parameters.getParameterNames(),
         co_freevars=(),
@@ -781,7 +781,7 @@ def addFunctionVariableReleases(function):
             continue
 
         releases.append(
-            StatementReleaseVariable(variable=variable, source_ref=source_ref)
+            makeStatementReleaseVariable(variable=variable, source_ref=source_ref)
         )
 
     if releases:
@@ -794,4 +794,4 @@ def addFunctionVariableReleases(function):
             provider=function, tried=body, final=releases, source_ref=source_ref
         )
 
-        function.setChild("body", makeStatementsSequenceFromStatement(statement=body))
+        function.setChildBody(makeStatementsSequenceFromStatement(statement=body))
