@@ -30,8 +30,8 @@ from nuitka.nodes.FrameNodes import (
     StatementsFrameGenerator,
 )
 from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCreation,
     ExpressionFunctionRef,
+    makeExpressionFunctionCreation,
 )
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
@@ -53,7 +53,7 @@ from .ReformulationFunctionStatements import (
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
     buildNode,
-    buildNodeList,
+    buildNodeTuple,
     detectFunctionBodyKind,
     getKind,
     makeStatementsSequenceFromStatement,
@@ -92,8 +92,7 @@ def buildLambdaNode(provider, node, source_ref):
         code_body.qualname_provider = provider
 
     if function_kind == "Generator":
-        function_body.setChild(
-            "body",
+        function_body.setChildBody(
             makeStatementsSequenceFromStatement(
                 statement=StatementReturn(
                     expression=ExpressionMakeGeneratorObject(
@@ -104,10 +103,10 @@ def buildLambdaNode(provider, node, source_ref):
                     ),
                     source_ref=source_ref,
                 )
-            ),
+            )
         )
 
-    defaults = buildNodeList(provider, node.args.defaults, source_ref)
+    defaults = buildNodeTuple(provider, node.args.defaults, source_ref)
     kw_defaults = buildParameterKwDefaults(
         provider=provider, node=node, function_body=function_body, source_ref=source_ref
     )
@@ -171,11 +170,11 @@ def buildLambdaNode(provider, node, source_ref):
 
     body = makeStatementsSequenceFromStatement(statement=body)
 
-    code_body.setChild("body", body)
+    code_body.setChildBody(body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
 
-    return ExpressionFunctionCreation(
+    return makeExpressionFunctionCreation(
         function_ref=ExpressionFunctionRef(
             function_body=outer_body, source_ref=source_ref
         ),
