@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,15 +23,11 @@ nodes that build tuples, lists, or sets.
 For Python3.5, unpacking can happen while creating sequences, these are
 being re-formulated to an internal function.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
 from nuitka.nodes.BuiltinNextNodes import ExpressionBuiltinNext1
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinTuple
@@ -42,23 +38,25 @@ from nuitka.nodes.ContainerMakingNodes import (
     makeExpressionMakeTuple,
     makeExpressionMakeTupleOrConstant,
 )
-from nuitka.nodes.ContainerOperationNodes import (
+from nuitka.nodes.ContainerOperationNodes import ExpressionSetOperationUpdate
+from nuitka.nodes.FunctionNodes import (
+    ExpressionFunctionRef,
+    makeExpressionFunctionCall,
+    makeExpressionFunctionCreation,
+)
+from nuitka.nodes.ListOperationNodes import (
     ExpressionListOperationExtend,
     ExpressionListOperationExtendForUnpack,
-    ExpressionSetOperationUpdate,
-)
-from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCall,
-    ExpressionFunctionCreation,
-    ExpressionFunctionRef,
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
 )
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.PythonVersions import python_version
 from nuitka.specs.ParameterSpecs import ParameterSpec
 
@@ -72,7 +70,7 @@ from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
     buildNode,
-    buildNodeList,
+    buildNodeTuple,
     getKind,
     makeStatementsSequenceFromStatement,
     makeStatementsSequenceFromStatements,
@@ -87,10 +85,10 @@ def _raiseStarredSyntaxError(element, source_ref):
 
 
 def buildTupleCreationNode(provider, node, source_ref):
-    if python_version >= 300:
+    if python_version >= 0x300:
         for element in node.elts:
             if getKind(element) == "Starred":
-                if python_version < 350:
+                if python_version < 0x350:
                     _raiseStarredSyntaxError(element, source_ref)
                 else:
                     return buildTupleUnpacking(
@@ -98,17 +96,17 @@ def buildTupleCreationNode(provider, node, source_ref):
                     )
 
     return makeExpressionMakeTupleOrConstant(
-        elements=buildNodeList(provider, node.elts, source_ref),
+        elements=buildNodeTuple(provider, node.elts, source_ref),
         user_provided=True,
         source_ref=source_ref,
     )
 
 
 def buildListCreationNode(provider, node, source_ref):
-    if python_version >= 300:
+    if python_version >= 0x300:
         for element in node.elts:
             if getKind(element) == "Starred":
-                if python_version < 350:
+                if python_version < 0x350:
                     _raiseStarredSyntaxError(element, source_ref)
                 else:
                     return buildListUnpacking(
@@ -116,17 +114,17 @@ def buildListCreationNode(provider, node, source_ref):
                     )
 
     return makeExpressionMakeListOrConstant(
-        elements=buildNodeList(provider, node.elts, source_ref),
+        elements=buildNodeTuple(provider, node.elts, source_ref),
         user_provided=True,
         source_ref=source_ref,
     )
 
 
 def buildSetCreationNode(provider, node, source_ref):
-    if python_version >= 300:
+    if python_version >= 0x300:
         for element in node.elts:
             if getKind(element) == "Starred":
-                if python_version < 350:
+                if python_version < 0x350:
                     _raiseStarredSyntaxError(element, source_ref)
                 else:
                     return _buildSetUnpacking(
@@ -134,7 +132,7 @@ def buildSetCreationNode(provider, node, source_ref):
                     )
 
     return makeExpressionMakeSetLiteralOrConstant(
-        elements=buildNodeList(provider, node.elts, source_ref),
+        elements=buildNodeTuple(provider, node.elts, source_ref),
         user_provided=True,
         source_ref=source_ref,
     )
@@ -163,14 +161,14 @@ def getListUnpackingHelper():
     tmp_iter_variable = result.allocateTempVariable(temp_scope, "iter")
     tmp_item_variable = result.allocateTempVariable(temp_scope, "keys")
 
-    if python_version < 390:
+    if python_version < 0x390:
         list_operation_extend = ExpressionListOperationExtend
     else:
         list_operation_extend = ExpressionListOperationExtendForUnpack
 
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_item_variable,
                 source=ExpressionBuiltinNext1(
                     value=ExpressionTempVariableRef(
@@ -201,19 +199,19 @@ def getListUnpackingHelper():
     args_variable = result.getVariableForAssignment(variable_name="args")
 
     final = (
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_result_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_item_variable, source_ref=internal_source_ref
         ),
     )
 
     tried = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionVariableRef(
@@ -223,12 +221,12 @@ def getListUnpackingHelper():
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_result_variable,
             source=makeConstantRefNode(constant=[], source_ref=internal_source_ref),
             source_ref=internal_source_ref,
         ),
-        StatementLoop(body=loop_body, source_ref=internal_source_ref),
+        StatementLoop(loop_body=loop_body, source_ref=internal_source_ref),
         StatementReturn(
             expression=ExpressionTempVariableRef(
                 variable=tmp_result_variable, source_ref=internal_source_ref
@@ -237,7 +235,7 @@ def getListUnpackingHelper():
         ),
     )
 
-    result.setBody(
+    result.setChildBody(
         makeStatementsSequenceFromStatement(
             makeTryFinallyStatement(
                 provider=result,
@@ -276,7 +274,7 @@ def getSetUnpackingHelper():
 
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_item_variable,
                 source=ExpressionBuiltinNext1(
                     value=ExpressionTempVariableRef(
@@ -307,19 +305,19 @@ def getSetUnpackingHelper():
     args_variable = result.getVariableForAssignment(variable_name="args")
 
     final = (
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_result_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
-        StatementReleaseVariable(
+        makeStatementReleaseVariable(
             variable=tmp_item_variable, source_ref=internal_source_ref
         ),
     )
 
     tried = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionVariableRef(
@@ -329,12 +327,12 @@ def getSetUnpackingHelper():
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_result_variable,
             source=makeConstantRefNode(constant=set(), source_ref=internal_source_ref),
             source_ref=internal_source_ref,
         ),
-        StatementLoop(body=loop_body, source_ref=internal_source_ref),
+        StatementLoop(loop_body=loop_body, source_ref=internal_source_ref),
         StatementReturn(
             expression=ExpressionTempVariableRef(
                 variable=tmp_result_variable, source_ref=internal_source_ref
@@ -343,7 +341,7 @@ def getSetUnpackingHelper():
         ),
     )
 
-    result.setBody(
+    result.setChildBody(
         makeStatementsSequenceFromStatement(
             makeTryFinallyStatement(
                 provider=result,
@@ -377,8 +375,8 @@ def buildListUnpacking(provider, elements, source_ref):
                 )
             )
 
-    result = ExpressionFunctionCall(
-        function=ExpressionFunctionCreation(
+    result = makeExpressionFunctionCall(
+        function=makeExpressionFunctionCreation(
             function_ref=ExpressionFunctionRef(
                 function_body=getListUnpackingHelper(), source_ref=source_ref
             ),
@@ -387,7 +385,7 @@ def buildListUnpacking(provider, elements, source_ref):
             annotations=None,
             source_ref=source_ref,
         ),
-        values=(makeExpressionMakeTuple(helper_args, source_ref),),
+        values=(makeExpressionMakeTuple(tuple(helper_args), source_ref),),
         source_ref=source_ref,
     )
 
@@ -422,8 +420,8 @@ def _buildSetUnpacking(provider, elements, source_ref):
                 )
             )
 
-    result = ExpressionFunctionCall(
-        function=ExpressionFunctionCreation(
+    result = makeExpressionFunctionCall(
+        function=makeExpressionFunctionCreation(
             function_ref=ExpressionFunctionRef(
                 function_body=getSetUnpackingHelper(), source_ref=source_ref
             ),
@@ -432,7 +430,7 @@ def _buildSetUnpacking(provider, elements, source_ref):
             annotations=None,
             source_ref=source_ref,
         ),
-        values=(makeExpressionMakeTuple(helper_args, source_ref),),
+        values=(makeExpressionMakeTuple(tuple(helper_args), source_ref),),
         source_ref=source_ref,
     )
 
