@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,20 +17,14 @@
 #
 """ Reformulation of with statements.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
 from nuitka import Options
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
-from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
-    ExpressionAttributeLookupSpecial,
-)
+from nuitka.nodes.AttributeLookupNodes import ExpressionAttributeLookupSpecial
+from nuitka.nodes.AttributeNodes import makeExpressionAttributeLookup
 from nuitka.nodes.CallNodes import (
     ExpressionCallEmpty,
     ExpressionCallNoKeywords,
@@ -38,7 +32,7 @@ from nuitka.nodes.CallNodes import (
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import makeStatementConditional
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
+from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTuple
 from nuitka.nodes.CoroutineNodes import (
     ExpressionAsyncWaitEnter,
     ExpressionAsyncWaitExit,
@@ -52,12 +46,16 @@ from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence,
 )
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.nodes.YieldNodes import ExpressionYieldFromWaitable
 from nuitka.PythonVersions import python_version
 
 from .ReformulationAssignmentStatements import buildAssignmentStatements
-from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNodeWithPublish
+from .ReformulationTryExceptStatements import (
+    makeTryExceptSingleHandlerNodeWithPublish,
+)
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
     buildNode,
@@ -68,10 +66,10 @@ from .TreeHelpers import (
 
 
 def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref):
-    # Many details, pylint: disable=too-many-locals
+    # Many details, pylint: disable=too-many-branches,too-many-locals
     with_source = buildNode(provider, context_expr, source_ref)
 
-    if python_version < 380 and Options.isFullCompat():
+    if python_version < 0x380 and Options.is_full_compat:
         source_ref = with_source.getCompatibleSourceReference()
 
     temp_scope = provider.allocateTempScope("with")
@@ -108,13 +106,13 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
         statements=statements, allow_none=True, source_ref=source_ref
     )
 
-    if body:
+    if body and python_version < 0x3A0:
         deepest = body
 
         while deepest.getVisitableNodes():
             deepest = deepest.getVisitableNodes()[-1]
 
-        if python_version < 370:
+        if python_version < 0x370:
             body_lineno = deepest.getCompatibleSourceReference().getLineNumber()
         else:
             body_lineno = deepest.getSourceReference().getLineNumber()
@@ -125,13 +123,13 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
 
     # The "__enter__" and "__exit__" were normal attribute lookups under
     # CPython2.6, but that changed with CPython2.7.
-    if python_version < 270:
-        attribute_lookup_class = ExpressionAttributeLookup
+    if python_version < 0x270:
+        attribute_lookup_maker = makeExpressionAttributeLookup
     else:
-        attribute_lookup_class = ExpressionAttributeLookupSpecial
+        attribute_lookup_maker = ExpressionAttributeLookupSpecial
 
     enter_value = ExpressionCallEmpty(
-        called=attribute_lookup_class(
+        called=attribute_lookup_maker(
             expression=ExpressionTempVariableRef(
                 variable=tmp_source_variable, source_ref=source_ref
             ),
@@ -145,7 +143,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
         called=ExpressionTempVariableRef(
             variable=tmp_exit_variable, source_ref=with_exit_source_ref
         ),
-        args=ExpressionMakeTuple(
+        args=makeExpressionMakeTuple(
             elements=(
                 ExpressionCaughtExceptionTypeRef(source_ref=with_exit_source_ref),
                 ExpressionCaughtExceptionValueRef(source_ref=with_exit_source_ref),
@@ -166,12 +164,6 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
 
     # For "async with", await the entered value and exit value must be awaited.
     if not sync:
-        enter_value = ExpressionYieldFromWaitable(
-            expression=ExpressionAsyncWaitEnter(
-                expression=enter_value, source_ref=source_ref
-            ),
-            source_ref=source_ref,
-        )
         exit_value_exception = ExpressionYieldFromWaitable(
             expression=ExpressionAsyncWaitExit(
                 expression=exit_value_exception, source_ref=source_ref
@@ -185,39 +177,72 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
             source_ref=source_ref,
         )
 
+    # First assign the with context to a temporary variable.
     statements = [
-        # First assign the with context to a temporary variable.
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_source_variable, source=with_source, source_ref=source_ref
         )
     ]
 
-    attribute_assignments = [
-        # Next, assign "__enter__" and "__exit__" attributes to temporary
-        # variables.
-        StatementAssignmentVariable(
-            variable=tmp_exit_variable,
-            source=attribute_lookup_class(
-                expression=ExpressionTempVariableRef(
-                    variable=tmp_source_variable, source_ref=source_ref
+    # Before 3.9, __aenter__ is immediately awaited, after we first do __aexit__ lookup.
+    if not sync and python_version < 0x390:
+        enter_value = ExpressionYieldFromWaitable(
+            expression=ExpressionAsyncWaitEnter(
+                expression=enter_value, source_ref=source_ref
+            ),
+            source_ref=source_ref,
+        )
+
+    attribute_enter_assignment = makeStatementAssignmentVariable(
+        variable=tmp_enter_variable, source=enter_value, source_ref=source_ref
+    )
+
+    attribute_exit_assignment = makeStatementAssignmentVariable(
+        variable=tmp_exit_variable,
+        source=attribute_lookup_maker(
+            expression=ExpressionTempVariableRef(
+                variable=tmp_source_variable, source_ref=source_ref
+            ),
+            attribute_name="__exit__" if sync else "__aexit__",
+            source_ref=source_ref,
+        ),
+        source_ref=source_ref,
+    )
+
+    # Next, assign "__enter__" and "__exit__" attributes to temporary variables, and
+    # depending on Python versions switch the order of these lookups and the order of
+    # awaiting enter.
+    # Normal "with" statements are enter, exit ordered after 3.6, and "async with"
+    # are since 3.9, and since 3.9 the enter is not awaited, until an exit is present.
+    if python_version >= 0x390 and not sync:
+        enter_await_statement = makeStatementAssignmentVariable(
+            variable=tmp_enter_variable,
+            source=ExpressionYieldFromWaitable(
+                expression=ExpressionAsyncWaitEnter(
+                    expression=ExpressionTempVariableRef(
+                        variable=tmp_enter_variable, source_ref=source_ref
+                    ),
+                    source_ref=source_ref,
                 ),
-                attribute_name="__exit__" if sync else "__aexit__",
                 source_ref=source_ref,
             ),
             source_ref=source_ref,
-        ),
-        StatementAssignmentVariable(
-            variable=tmp_enter_variable, source=enter_value, source_ref=source_ref
-        ),
-    ]
+        )
 
-    if python_version >= 360 and sync:
-        attribute_assignments.reverse()
+        attribute_assignments = (
+            attribute_enter_assignment,
+            attribute_exit_assignment,
+            enter_await_statement,
+        )
+    elif python_version >= 0x360 and sync:
+        attribute_assignments = (attribute_enter_assignment, attribute_exit_assignment)
+    else:
+        attribute_assignments = (attribute_exit_assignment, attribute_enter_assignment)
 
-    statements += attribute_assignments
+    statements.extend(attribute_assignments)
 
     statements.append(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_indicator_variable,
             source=makeConstantRefNode(constant=True, source_ref=source_ref),
             source_ref=source_ref,
@@ -235,7 +260,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
                     statements=(
                         # Prevents final block from calling __exit__ as
                         # well.
-                        StatementAssignmentVariable(
+                        makeStatementAssignmentVariable(
                             variable=tmp_indicator_variable,
                             source=makeConstantRefNode(
                                 constant=False, source_ref=source_ref
@@ -253,7 +278,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
                     ),
                     source_ref=source_ref,
                 ),
-                public_exc=python_version >= 270,
+                public_exc=python_version >= 0x270,
                 source_ref=source_ref,
             ),
             final=makeStatementConditional(
@@ -278,13 +303,13 @@ def _buildWithNode(provider, context_expr, assign_target, body, sync, source_ref
         provider=provider,
         tried=statements,
         final=(
-            StatementReleaseVariable(
+            makeStatementReleaseVariable(
                 variable=tmp_source_variable, source_ref=with_exit_source_ref
             ),
-            StatementReleaseVariable(
+            makeStatementReleaseVariable(
                 variable=tmp_enter_variable, source_ref=with_exit_source_ref
             ),
-            StatementReleaseVariable(
+            makeStatementReleaseVariable(
                 variable=tmp_exit_variable, source_ref=with_exit_source_ref
             ),
         ),
