@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,11 +24,6 @@ Only after this is executed, variable reference nodes can be considered
 complete.
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementDelVariable,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.FunctionNodes import MaybeLocalVariableUsage
 from nuitka.nodes.LocalsDictNodes import (
     ExpressionLocalsVariableRef,
@@ -41,10 +36,13 @@ from nuitka.nodes.NodeMakingHelpers import (
     mergeStatements,
 )
 from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableDelNodes import makeStatementDelVariable
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     makeExpressionVariableRef,
 )
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 from nuitka.PythonVersions import (
     getErrorMessageExecWithNestedFunction,
     python_version,
@@ -87,17 +85,19 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             source_ref,
         ) in node.consumeNonlocalDeclarations():
             for non_local_name in non_local_names:
-
                 variable = node.takeVariableForClosure(variable_name=non_local_name)
 
-                node.getLocalsScope().registerClosureVariable(variable)
-
                 if variable.isModuleVariable() and user_provided:
                     raiseSyntaxError(
                         "no binding for nonlocal '%s' found" % (non_local_name),
                         source_ref,
                     )
 
+                if node.isExpressionClassBody() and non_local_name == "__class__":
+                    pass
+                else:
+                    node.getLocalsScope().registerClosureVariable(variable)
+
                 variable.addVariableUser(node)
 
     @staticmethod
@@ -117,7 +117,9 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                         qualname_node = qualname_assign.subnode_source
 
                         new_node = makeConstantReplacementNode(
-                            constant=class_variable.getName(), node=qualname_node
+                            constant=class_variable.getName(),
+                            node=qualname_node,
+                            user_provided=True,
                         )
 
                         parent = qualname_node.parent
@@ -165,22 +167,22 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
                     statements = mergeStatements(
                         statements=(
-                            StatementAssignmentVariable(
+                            makeStatementAssignmentVariable(
                                 variable=tmp_variable,
-                                source=node.subnode_source.getLeft(),
+                                source=node.subnode_source.subnode_left,
                                 source_ref=node.source_ref,
                             ),
                             makeTryFinallyStatement(
                                 provider=provider,
                                 tried=(
-                                    StatementAssignmentVariable(
+                                    makeStatementAssignmentVariable(
                                         variable=tmp_variable,
                                         source=makeExpressionOperationBinaryInplace(
                                             left=ExpressionTempVariableRef(
                                                 variable=tmp_variable,
                                                 source_ref=node.source_ref,
                                             ),
-                                            right=node.subnode_source.getRight(),
+                                            right=node.subnode_source.subnode_right,
                                             operator=node.subnode_source.getOperator(),
                                             source_ref=node.source_ref,
                                         ),
@@ -189,14 +191,14 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                                     StatementLocalsDictOperationSet(
                                         locals_scope=provider.getLocalsScope(),
                                         variable_name=variable_name,
-                                        value=ExpressionTempVariableRef(
+                                        source=ExpressionTempVariableRef(
                                             variable=tmp_variable,
                                             source_ref=node.source_ref,
                                         ),
                                         source_ref=node.source_ref,
                                     ),
                                 ),
-                                final=StatementReleaseVariable(
+                                final=makeStatementReleaseVariable(
                                     variable=tmp_variable, source_ref=node.source_ref
                                 ),
                                 source_ref=node.source_ref,
@@ -210,7 +212,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     new_node = StatementLocalsDictOperationSet(
                         locals_scope=provider.getLocalsScope(),
                         variable_name=variable_name,
-                        value=node.subnode_source,
+                        source=node.subnode_source,
                         source_ref=node.source_ref,
                     )
 
@@ -220,7 +222,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     variable_name=variable_name
                 )
 
-                new_node = StatementAssignmentVariable(
+                new_node = makeStatementAssignmentVariable(
                     variable=variable,
                     source=node.subnode_source,
                     source_ref=node.source_ref,
@@ -251,7 +253,7 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     variable_name=variable_name
                 )
 
-                new_node = StatementDelVariable(
+                new_node = makeStatementDelVariable(
                     variable=variable,
                     tolerant=node.tolerant,
                     source_ref=node.source_ref,
@@ -304,11 +306,11 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             if node.getVariable().getOwner() != node.getParentVariableProvider():
                 node.getParentVariableProvider().addClosureVariable(node.getVariable())
         elif node.isExpressionGeneratorObjectBody():
-            if python_version >= 300:
+            if python_version >= 0x300:
                 self._handleNonLocal(node)
 
             # Only Python3.4 or later allows for generators to have qualname.
-            if python_version >= 340:
+            if python_version >= 0x340:
                 self._handleQualnameSetup(node)
         elif node.isExpressionCoroutineObjectBody():
             self._handleNonLocal(node)
@@ -319,20 +321,20 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
             self._handleQualnameSetup(node)
         elif node.isExpressionClassBody():
-            if python_version >= 300:
+            if python_version >= 0x300:
                 self._handleNonLocal(node)
 
             # Python3.4 allows for class declarations to be made global, even
             # after they were declared, so we need to fix this up.
-            if python_version >= 340:
+            if python_version >= 0x340:
                 self._handleQualnameSetup(node)
         elif node.isExpressionFunctionBody():
-            if python_version >= 300:
+            if python_version >= 0x300:
                 self._handleNonLocal(node)
 
             # Python 3.4 allows for class declarations to be made global, even
             # after they were declared, so we need to fix this up.
-            if python_version >= 340:
+            if python_version >= 0x340:
                 self._handleQualnameSetup(node)
         # Check if continue and break are properly in loops. If not, raise a
         # syntax error.
@@ -374,7 +376,7 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
         variable = provider.getVariableForReference(variable_name=variable_name)
 
         # Need to catch functions with "exec" and closure variables not allowed.
-        if python_version < 300 and provider.isExpressionFunctionBodyBase():
+        if python_version < 0x300 and provider.isExpressionFunctionBodyBase():
             was_taken = provider.hasTakenVariable(variable_name)
 
             if not was_taken and variable.getOwner() is not provider:
@@ -444,7 +446,7 @@ class VariableClosureLookupVisitorPhase3(VisitorNoopMixin):
     """
 
     def onEnterNode(self, node):
-        if python_version < 300 and node.isStatementDelVariable():
+        if python_version < 0x300 and node.isStatementDelVariable():
             variable = node.getVariable()
 
             if not variable.isModuleVariable() and isSharedAmongScopes(variable):
