@@ -206,12 +206,7 @@ def _getRealPathWindows(path):
                 [
                     _powershell_path,
                     "-NoProfile",
-                    "Get-Item",
-                    path,
-                    "|",
-                    "Select-Object",
-                    "-ExpandProperty",
-                    "Target",
+                    'Get-Item "%s" | Select-Object -ExpandProperty Target' % path,
                 ],
                 shell=False,
             )
@@ -306,7 +301,7 @@ def getFileList(
 
     result = []
 
-    # Normalize ignoredirs for better matching.
+    # Normalize "ignore_dirs" for better matching.
     ignore_dirs = [os.path.normcase(ignore_dir) for ignore_dir in ignore_dirs]
     ignore_filenames = [
         os.path.normcase(ignore_filename) for ignore_filename in ignore_filenames
@@ -802,6 +797,23 @@ def copyFileWithPermissions(source_path, dest_path):
         os.chmod(dest_path, source_mode)
 
 
+def queryUser(question, choices, default, default_non_interactive):
+    assert default in choices, (default, choices)
+    assert default_non_interactive in choices, (default, choices)
+
+    prompt = "%s? (%s)" % (
+        question,
+        "/".join(choice.upper() if choice == default else choice for choice in choices),
+    )
+
+    try:
+        reply = raw_input(prompt) or default
+    except EOFError:
+        reply = default_non_interactive
+
+    return reply.lower()
+
+
 def copyFile(source_path, dest_path):
     """Improved version of shutil.copy
 
@@ -818,12 +830,15 @@ def copyFile(source_path, dest_path):
 
             general.warning("Problem copying file %s:" % e)
 
-            try:
-                reply = raw_input("Retry? (YES/no) ") or "yes"
-            except EOFError:
-                reply = "no"
-
-            if reply.upper() == "YES":
+            if (
+                queryUser(
+                    "Retry?",
+                    choices=("yes", "no"),
+                    default="yes",
+                    default_non_interactive="no",
+                )
+                == "yes"
+            ):
                 continue
 
             raise
@@ -838,8 +853,8 @@ def getWindowsDrive(path):
     return os.path.normcase(drive)
 
 
-def isPathBelow(path, filename):
-    """Is a path inside of a given directory path
+def isFilenameBelowPath(path, filename):
+    """Is a filename inside of a given directory path
 
     Args:
         path: location to be below
@@ -847,7 +862,7 @@ def isPathBelow(path, filename):
     """
     if type(path) in (tuple, list):
         for p in path:
-            if isPathBelow(path=p, filename=filename):
+            if isFilenameBelowPath(path=p, filename=filename):
                 return True
 
         return False
@@ -862,9 +877,9 @@ def isPathBelow(path, filename):
     return os.path.relpath(filename, path).split(os.path.sep)[0] != ".."
 
 
-def isPathBelowOrSameAs(path, filename):
-    """Is a path inside of a given directory path or the same path as that directory."""
-    return isPathBelow(path, filename) or areSamePaths(path, filename)
+def isFilenameSameAsOrBelowPath(path, filename):
+    """Is a filename inside of a given directory path or the same path as that directory."""
+    return isFilenameBelowPath(path, filename) or areSamePaths(path, filename)
 
 
 def getWindowsShortPathName(filename):
@@ -880,11 +895,11 @@ def getWindowsShortPathName(filename):
     import ctypes.wintypes
 
     GetShortPathNameW = ctypes.windll.kernel32.GetShortPathNameW
-    GetShortPathNameW.argtypes = [
+    GetShortPathNameW.argtypes = (
         ctypes.wintypes.LPCWSTR,
         ctypes.wintypes.LPWSTR,
         ctypes.wintypes.DWORD,
-    ]
+    )
     GetShortPathNameW.restype = ctypes.wintypes.DWORD
 
     output_buf_size = 0
@@ -941,6 +956,17 @@ def getExternalUsePath(filename, only_dirname=False):
     return filename
 
 
+def getReportPath(filename):
+    """Convert filename into a path suitable for reporting, avoiding home directory paths."""
+    if os.path.isabs(os.path.expanduser(filename)):
+        abs_filename = os.path.abspath(os.path.expanduser(filename))
+        home_path = os.path.expanduser("~")
+        if isFilenameBelowPath(path=home_path, filename=abs_filename):
+            return os.path.join("~", relpath(path=abs_filename, start=home_path))
+
+    return filename
+
+
 def getLinkTarget(filename):
     """Return the path a link is pointing too, if any.
 
@@ -1024,3 +1050,14 @@ def withDirectoryChange(path, allow_none=False):
 
     if path is not None or not allow_none:
         os.chdir(old_cwd)
+
+
+def containsPathElements(path, elements):
+    """Test if a path contains any unwanted elements."""
+
+    elements = tuple(os.path.normcase(element) for element in elements)
+    path = os.path.normpath(path)
+
+    parts = os.path.normpath(path).split(os.path.sep)
+
+    return any(element in parts for element in elements)
